@@ -1,8 +1,8 @@
 /*
- * $Id: abox.c,v 1.20 2004/07/30 17:08:38 tal197 Exp $
+ * $Id: abox.c,v 1.27 2005/09/13 17:06:48 kerofin Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2003, the ROX-Filer team.
+ * Copyright (C) 2005, the ROX-Filer team.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -187,6 +187,7 @@ static void abox_init(GTypeInstance *object, gpointer gclass)
 				1, 2, i, i + 1,
 				GTK_SHRINK, GTK_SHRINK, 4, 1);
 		abox->cmp_name[i] = gtk_label_new("");
+		gtk_label_set_line_wrap(GTK_LABEL(abox->cmp_name[i]), TRUE);
 		gtk_misc_set_alignment(GTK_MISC(abox->cmp_name[i]), 0., 0.5);
 		gtk_table_attach(GTK_TABLE(abox->cmp_area),
 				abox->cmp_name[i],
@@ -210,6 +211,8 @@ static void abox_init(GTypeInstance *object, gpointer gclass)
 				0, 1, 0, 2,
 				GTK_SHRINK, GTK_EXPAND | GTK_FILL, 1, 2);
 
+	abox->progress=NULL;
+
 	abox->flag_box = gtk_hbox_new(FALSE, 16);
 	gtk_box_pack_end(GTK_BOX(dialog->vbox),
 				abox->flag_box, FALSE, TRUE, 2);
@@ -330,7 +333,7 @@ static void abox_finalise(GObject *object)
 {
 	GObjectClass *parent_class;
 	ABox *abox = ABOX(object);
-	
+
 	if (abox->next_dir)
 	{
 		null_g_free(&abox->next_dir);
@@ -347,6 +350,8 @@ static gboolean show_next_dir(gpointer data)
 {
 	ABox	*abox = (ABox *) data;
 
+	g_return_val_if_fail(IS_ABOX(abox), FALSE);
+
 	gtk_label_set_text(GTK_LABEL(abox->dir_label), abox->next_dir);
 	null_g_free(&abox->next_dir);
 	
@@ -465,6 +470,8 @@ void abox_add_results(ABox *abox)
 
 	g_signal_connect(abox->results, "row-activated",
 			G_CALLBACK(select_row_callback), abox);
+
+	gtk_widget_show_all(frame);
 }
 
 void abox_add_filename(ABox *abox, const gchar *path)
@@ -613,12 +620,12 @@ void abox_set_file(ABox *abox, int i, const gchar *path)
 {
 	DirItem *item;
 	gchar *base;
-	
+
 	g_return_if_fail(i >= 0 && i < 2);
+	g_return_if_fail(IS_ABOX(abox));
 
 	if (!path || !path[0])
 	{
-		/*printf("path=%s\n", path? path: "NULL");*/
 		gtk_widget_hide(abox->cmp_icon[i]);
 		gtk_widget_hide(abox->cmp_name[i]);
 		gtk_widget_hide(abox->cmp_size[i]);
@@ -633,7 +640,7 @@ void abox_set_file(ABox *abox, int i, const gchar *path)
 	diritem_restat(path, item, NULL);
 
 	gtk_image_set_from_pixbuf(GTK_IMAGE(abox->cmp_icon[i]),
-				  item->image->pixbuf);
+				  di_image(item)->pixbuf);
 	gtk_widget_show(abox->cmp_icon[i]);
 
 	gtk_label_set_text(GTK_LABEL(abox->cmp_name[i]), item->leafname);
@@ -664,3 +671,21 @@ void abox_set_file(ABox *abox, int i, const gchar *path)
 	diritem_free(item);
 }
 
+void    abox_set_percentage(ABox *abox, int per)
+{
+	if(!abox->progress) {
+		GtkDialog *dialog = GTK_DIALOG(abox);
+		
+		abox->progress=gtk_progress_bar_new ();
+		gtk_box_pack_start(GTK_BOX(dialog->vbox),
+				abox->progress, FALSE, FALSE, 2);
+		gtk_widget_show(abox->progress);
+	}
+	if(per<0 || per>100) {
+		gtk_widget_hide(abox->progress);
+		return;
+	}
+	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(abox->progress),
+				      per/100.);
+}
+
