@@ -1,8 +1,8 @@
 /*
- * $Id: action.c,v 1.154 2004/07/30 17:08:38 tal197 Exp $
+ * $Id: action.c,v 1.166 2005/09/13 17:06:11 kerofin Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2003, the ROX-Filer team.
+ * Copyright (C) 2005, the ROX-Filer team.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -54,6 +54,14 @@
 #include "type.h"
 #include "xtypes.h"
 
+#if defined(HAVE_GETXATTR)
+# define ATTR_MAN_PAGE N_("See the attr(5) man page for full details.")
+#elif defined(HAVE_ATTROPEN)
+# define ATTR_MAN_PAGE N_("See the fsattr(5) man page for full details.")
+#else
+# define ATTR_MAN_PAGE N_("You do not appear to have OS support.")
+#endif 
+
 /* Parent->Child messages are one character each:
  *
  * Y/N 		Yes/No button clicked
@@ -89,6 +97,7 @@ struct _GUIside
  * using them again.
  */
 static gboolean mount_open_dir = FALSE;
+static gboolean mount_mount = FALSE;	/* (FALSE => unmount) */
 static int 	from_parent = 0;
 static FILE	*to_parent = NULL;
 static gboolean	quiet = FALSE;
@@ -132,7 +141,7 @@ static void send_done(void);
 static void send_check_path(const gchar *path);
 static void send_mount_path(const gchar *path);
 static gboolean printf_send(const char *msg, ...);
-static gboolean send(void);
+static gboolean send_msg(void);
 static gboolean send_error(void);
 static gboolean send_dir(const char *dir);
 static gboolean read_exact(int source, char *buffer, ssize_t len);
@@ -234,7 +243,7 @@ static void show_chmod_help(gpointer data)
 	GtkWidget *text;
 
 	help = gtk_dialog_new_with_buttons(
-			_("Find expression reference"),
+			_("Change permissions reference"),
 			NULL, 0,
 			GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL,
 			NULL);
@@ -308,9 +317,11 @@ static void show_settype_help(gpointer data)
 "\n"
 "File types are only supported for regular files, not\n"
 "directories, devices, pipes or sockets, and then only\n"
-"on certain file systems and where the OS implements them.\n" 
-"\n"
-ATTR_MAN_PAGE));
+"on certain file systems and where the OS implements them.\n"));
+
+	text = gtk_label_new(_(ATTR_MAN_PAGE));
+	gtk_misc_set_padding(GTK_MISC(text), 2, 2);
+	gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(help)->vbox), text);
 
 	g_signal_connect(help, "response",
 			G_CALLBACK(gtk_widget_destroy), NULL);
@@ -363,6 +374,10 @@ static void process_message(GUIside *gui_side, const gchar *buffer)
 		abox_set_file(abox, 1, buffer+1);
 		abox_show_compare(abox, TRUE);
 	}
+	else if (*buffer == '%')
+	{
+		abox_set_percentage(abox, atoi(buffer+1));
+	}
 	else
 		abox_log(abox, buffer + 1, NULL);
 }
@@ -513,18 +528,18 @@ static gboolean printf_send(const char *msg, ...)
 	g_string_assign(message, tmp);
 	g_free(tmp);
 
-	return send();
+	return send_msg();
 }
 
 /* Send 'message' to our parent process. TRUE on success. */
-static gboolean send(void)
+static gboolean send_msg(void)
 {
 	char len_buffer[5];
 	ssize_t len;
 
 	g_return_val_if_fail(message->len < 0xffff, FALSE);
 
-	sprintf(len_buffer, "%04x", message->len);
+	sprintf(len_buffer, "%04" G_GSIZE_MODIFIER "x", message->len);
 	fwrite(len_buffer, 1, 4, to_parent);
 	len = fwrite(message->str, 1, message->len, to_parent);
 	fflush(to_parent);
@@ -658,7 +673,7 @@ static void check_flags(void)
 
 /* Read until the user sends a reply. If ignore_quiet is TRUE then
  * the user MUST click Yes or No, else treat quiet on as Yes.
- * If the user needs prompting then does send().
+ * If the user needs prompting then does send_msg().
  */
 static gboolean printf_reply(int fd, gboolean ignore_quiet,
 			     const char *msg, ...)
@@ -678,7 +693,7 @@ static gboolean printf_reply(int fd, gboolean ignore_quiet,
 	g_string_assign(message, tmp);
 	g_free(tmp);
 
-	send();
+	send_msg();
 
 	while (1)
 	{
@@ -905,12 +920,16 @@ static void do_delete(const char *src_path, const char *unused)
 					   : access(src_path, W_OK) != 0;
 	if (write_prot || !quiet)
 	{
+		int res;
+		
 		printf_send("<%s", src_path);
 		printf_send(">");
-		if (!printf_reply(from_parent, write_prot && !o_force,
+		res=printf_reply(from_parent, write_prot && !o_force,
 				  _("?Delete %s'%s'?"),
 				  write_prot ? _("WRITE-PROTECTED ") : "",
-				  src_path))
+				 src_path);
+		printf_send("<");
+		if (!res)
 			return;
 	}
 	else if (!o_brief)
@@ -956,11 +975,14 @@ static void do_eject(const char *path)
 
 	if (!quiet)
 	{
+		int res;
 		printf_send("<%s", path);
 		printf_send(">");
-		if (!printf_reply(from_parent, !o_force,
+		res=printf_reply(from_parent, !o_force,
 				  _("?Eject '%s'?"),
-				  path))
+				 path);
+		printf_send("<");
+		if (!res)
 			return;
 	}
 	else if (!o_brief)
@@ -1098,10 +1120,13 @@ static void do_chmod(const char *path, const char *unused)
 
 	if (!quiet)
 	{
+		int res;
 		printf_send("<%s", path);
 		printf_send(">");
-		if (!printf_reply(from_parent, FALSE,
-				  _("?Change permissions of '%s'?"), path))
+		res=printf_reply(from_parent, FALSE,
+				 _("?Change permissions of '%s'?"), path);
+		printf_send("<");
+		if (!res)
 			return;
 	}
 	else if (!o_brief)
@@ -1175,10 +1200,13 @@ static void do_settype(const char *path, const char *unused)
 
 	if (!quiet)
 	{
+		int res;
 		printf_send("<%s", path);
 		printf_send(">");
-		if (!printf_reply(from_parent, FALSE,
-				  _("?Change type of '%s'?"), path))
+		res=printf_reply(from_parent, FALSE,
+				 _("?Change type of '%s'?"), path);
+		printf_send("<");
+		if (!res)
 			return;
 	}
 
@@ -1536,14 +1564,9 @@ static void do_move(const char *path, const char *dest)
 	}
 }
 
-static void do_link(const char *path, const char *dest)
+/* Common code for do_link_relative() and do_link_absolute(). */
+static void do_link(const char *path, const char *dest_path)
 {
-	const char	*dest_path;
-
-	check_flags();
-
-	dest_path = make_dest_path(path, dest);
-
 	if (quiet)
 		printf_send(_("'Linking %s as %s\n"), path, dest_path);
 	else {
@@ -1560,6 +1583,26 @@ static void do_link(const char *path, const char *dest)
 		send_check_path(dest_path);
 }
 
+static void do_link_relative(const char *path, const char *dest)
+{
+	char *rel_path;
+	const char *dest_path;
+
+	dest_path = make_dest_path(path, dest);
+
+	check_flags();
+
+	rel_path = get_relative_path(dest_path, path);
+	do_link(rel_path, dest_path);
+	g_free(rel_path);
+}
+
+static void do_link_absolute(const char *path, const char *dest)
+{
+	check_flags();
+	do_link(path, make_dest_path(path, dest));
+}
+
 /* Mount/umount this item (depending on 'mount') */
 static void do_mount(const guchar *path, gboolean mount)
 {
@@ -1625,10 +1668,12 @@ static void usage_cb(gpointer data)
 {
 	GList *paths = (GList *) data;
 	double	total_size = 0;
+	int n, i, per;
 
+	n=g_list_length(paths);
 	dir_counter = file_counter = 0;
 
-	for (; paths; paths = paths->next)
+	for (i=0; paths; paths = paths->next, i++)
 	{
 		guchar	*path = (guchar *) paths->data;
 
@@ -1636,6 +1681,11 @@ static void usage_cb(gpointer data)
 
 		size_tally = 0;
 		
+		if(n>1 && i>0)
+		{
+			per=100*i/n;
+			printf_send("%%%d", per);
+		}
 		do_usage(path, NULL);
 
 		printf_send("'%s: %s\n",
@@ -1643,6 +1693,7 @@ static void usage_cb(gpointer data)
 			    format_double_size(size_tally));
 		total_size += size_tally;
 	}
+	printf_send("%%-1");
 
 	g_string_printf(message, _("'\nTotal: %s ("),
 			format_double_size(total_size));
@@ -1661,7 +1712,7 @@ static void usage_cb(gpointer data)
 				dir_counter == 1 ? _("directory")
 						 : _("directories"));
 	
-	send();
+	send_msg();
 }
 
 #ifdef DO_MOUNT_POINTS
@@ -1669,25 +1720,28 @@ static void mount_cb(gpointer data)
 {
 	GList 		*paths = (GList *) data;
 	gboolean	mount_points = FALSE;
+	int n, i, per;
 
-	for (; paths; paths = paths->next)
+	n=g_list_length(paths);
+	for (i=0; paths; paths = paths->next, i++)
 	{
 		guchar *path = (guchar *) paths->data;
 		guchar *target;
 
-		target = readlink_dup(path);
+		target = pathdup(path);
 		if (!target)
 			target = path;
 
-		if (mount_is_mounted(target, NULL, NULL))
+		if(n>1 && i>0)
 		{
-			mount_points = TRUE;
-			do_mount(target, FALSE);	/* Unmount */
+			per=100*i/n;
+			printf_send("%%%d", per);
 		}
-		else if (g_hash_table_lookup(fstab_mounts, target))
+		if (mount_is_mounted(target, NULL, NULL) ||
+		    g_hash_table_lookup(fstab_mounts, target))
 		{
 			mount_points = TRUE;
-			do_mount(target, TRUE);	/* Mount */
+			do_mount(target, mount_mount);	/* Mount */
 		}
 
 		if (target != path)
@@ -1717,15 +1771,22 @@ static guchar *dirname(guchar *path)
 static void delete_cb(gpointer data)
 {
 	GList	*paths = (GList *) data;
+	int n, i, per;
 
-	for (; paths; paths = paths->next)
+	n=g_list_length(paths);
+	for (i=0; paths; paths = paths->next, i++)
 	{
 		guchar	*path = (guchar *) paths->data;
 		guchar	*dir;
-		
+
 		dir = dirname(path);
 		send_dir(dir);
 
+		if(n>1 && i>0)
+		{
+			per=100*i/n;
+			printf_send("%%%d", per);
+		}
 		do_delete(path, dir);
 
 		g_free(dir);
@@ -1737,11 +1798,19 @@ static void delete_cb(gpointer data)
 static void eject_cb(gpointer data)
 {
 	GList	*paths = (GList *) data;
+	int n, i, per;
 
-	for (; paths; paths = paths->next)
+	n=g_list_length(paths);
+
+	for (i=0; paths; paths = paths->next, i++)
 	{
 		guchar	*path = (guchar *) paths->data;
 		
+		if(n>1 && i>0)
+		{
+			per=100*i/n;
+			printf_send("%%%d", per);
+		}
 		send_dir(path);
 
 		do_eject(path);
@@ -1778,12 +1847,20 @@ static void find_cb(gpointer data)
 static void chmod_cb(gpointer data)
 {
 	GList *paths = (GList *) data;
+	int n, i, per;
 
-	for (; paths; paths = paths->next)
+	n=g_list_length(paths);
+
+	for (i=0; paths; paths = paths->next, i++)
 	{
 		guchar	*path = (guchar *) paths->data;
 		struct stat info;
 
+		if(n>1 && i>0)
+		{
+			per=100*i/n;
+			printf_send("%%%d", per);
+		}
 		send_dir(path);
 
 		if (mc_stat(path, &info) != 0)
@@ -1801,12 +1878,20 @@ static void chmod_cb(gpointer data)
 static void settype_cb(gpointer data)
 {
 	GList *paths = (GList *) data;
+	int n, i, per;
 
-	for (; paths; paths = paths->next)
+	n=g_list_length(paths);
+
+	for (i=0; paths; paths = paths->next, i++)
 	{
 		guchar	*path = (guchar *) paths->data;
 		struct stat info;
 
+		if(n>1 && i>0)
+		{
+			per=100*i/n;
+			printf_send("%%%d", per);
+		}
 		send_dir(path);
 
 		if (mc_stat(path, &info) != 0)
@@ -1824,9 +1909,17 @@ static void settype_cb(gpointer data)
 static void list_cb(gpointer data)
 {
 	GList	*paths = (GList *) data;
+	int n, i, per;
 
-	for (; paths; paths = paths->next)
+	n=g_list_length(paths);
+
+	for (i=0; paths; paths = paths->next, i++)
 	{
+		if(n>1 && i>0)
+		{
+			per=100*i/n;
+			printf_send("%%%d", per);
+		}
 		send_dir((char *) paths->data);
 
 		action_do_func((char *) paths->data, action_dest);
@@ -1876,7 +1969,7 @@ void action_find(GList *paths)
 	gui_side->entry_string_func = set_find_string_colour;
 
 	number_of_windows++;
-	gtk_widget_show_all(abox);
+	gtk_widget_show(abox);
 }
 
 /* Count disk space used by selected items */
@@ -1892,6 +1985,8 @@ void action_usage(GList *paths)
 	}
 
 	abox = abox_new(_("Disk Usage"), TRUE);
+	if(paths && paths->next)
+		abox_set_percentage(ABOX(abox), 0);
 	
 	gui_side = start_action(abox, usage_cb, paths,
 					 o_action_force.int_value,
@@ -1913,7 +2008,7 @@ void action_usage(GList *paths)
  * If open_dir is TRUE and the dir is successfully mounted, open it.
  * quiet can be -1 for default.
  */
-void action_mount(GList	*paths, gboolean open_dir, int quiet)
+void action_mount(GList	*paths, gboolean open_dir, gboolean mount, int quiet)
 {
 #ifdef DO_MOUNT_POINTS
 	GUIside		*gui_side;
@@ -1923,8 +2018,11 @@ void action_mount(GList	*paths, gboolean open_dir, int quiet)
 	 	quiet = o_action_mount.int_value;
 
 	mount_open_dir = open_dir;
+	mount_mount = mount;
 
 	abox = abox_new(_("Mount / Unmount"), quiet);
+	if(paths && paths->next)
+		abox_set_percentage(ABOX(abox), 0);
 	gui_side = start_action(abox, mount_cb, paths,
 					 o_action_force.int_value,
 					 o_action_brief.int_value,
@@ -1952,6 +2050,8 @@ void action_delete(GList *paths)
 		return;
 
 	abox = abox_new(_("Delete"), o_action_delete.int_value);
+	if(paths && paths->next)
+		abox_set_percentage(ABOX(abox), 0);
 	gui_side = start_action(abox, delete_cb, paths,
 					 o_action_force.int_value,
 					 o_action_brief.int_value,
@@ -2009,6 +2109,8 @@ void action_chmod(GList *paths, gboolean force_recurse, const char *action)
 		new_entry_string = g_strdup(last_chmod_string);
 
 	abox = abox_new(_("Permissions"), FALSE);
+	if(paths && paths->next)
+		abox_set_percentage(ABOX(abox), 0);
 	gui_side = start_action(abox, chmod_cb, paths,
 				o_action_force.int_value,
 				o_action_brief.int_value,
@@ -2068,6 +2170,8 @@ void action_settype(GList *paths, gboolean force_recurse, const char *oldtype)
 		new_entry_string = g_strdup(last_settype_string);
 
 	abox = abox_new(_("Set type"), FALSE);
+	if(paths && paths->next)
+		abox_set_percentage(ABOX(abox), 0);
 	gui_side = start_action(abox, settype_cb, paths,
 				o_action_force.int_value,
 				o_action_brief.int_value,
@@ -2118,6 +2222,8 @@ void action_copy(GList *paths, const char *dest, const char *leaf, int quiet)
 	action_do_func = do_copy;
 
 	abox = abox_new(_("Copy"), quiet);
+	if(paths && paths->next)
+		abox_set_percentage(ABOX(abox), 0);
 	gui_side = start_action(abox, list_cb, paths,
 					 o_action_force.int_value,
 					 o_action_brief.int_value,
@@ -2154,6 +2260,8 @@ void action_move(GList *paths, const char *dest, const char *leaf, int quiet)
 	action_do_func = do_move;
 
 	abox = abox_new(_("Move"), quiet);
+	if(paths && paths->next)
+		abox_set_percentage(ABOX(abox), 0);
 	gui_side = start_action(abox, list_cb, paths,
 					 o_action_force.int_value,
 					 o_action_brief.int_value,
@@ -2174,17 +2282,22 @@ void action_move(GList *paths, const char *dest, const char *leaf, int quiet)
 }
 
 /* If leaf is NULL then the link will have the same name */
-/* XXX: No quiet option here? */
-void action_link(GList *paths, const char *dest, const char *leaf)
+void action_link(GList *paths, const char *dest, const char *leaf,
+		 gboolean relative)
 {
 	GtkWidget	*abox;
 	GUIside		*gui_side;
 
 	action_dest = dest;
 	action_leaf = leaf;
-	action_do_func = do_link;
+	if (relative)
+		action_do_func = do_link_relative;
+	else
+		action_do_func = do_link_absolute;
 
 	abox = abox_new(_("Link"), o_action_link.int_value);
+	if(paths && paths->next)
+		abox_set_percentage(ABOX(abox), 0);
 	gui_side = start_action(abox, list_cb, paths,
 					 o_action_force.int_value,
 					 o_action_brief.int_value,
@@ -2204,6 +2317,8 @@ void action_eject(GList *paths)
 	GtkWidget	*abox;
 
 	abox = abox_new(_("Eject"), TRUE);
+	if(paths && paths->next)
+		abox_set_percentage(ABOX(abox), 0);
 	gui_side = start_action(abox, eject_cb, paths,
 					 o_action_force.int_value,
 					 o_action_brief.int_value,
