@@ -1,8 +1,8 @@
 /*
- * $Id: cell_icon.c,v 1.9 2004/03/01 23:06:23 kerofin Exp $
+ * $Id: cell_icon.c,v 1.12 2005/12/16 20:57:53 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2003, the ROX-Filer team.
+ * Copyright (C) 2005, the ROX-Filer team.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -215,7 +215,7 @@ static DisplayStyle get_style(GtkCellRenderer *cell)
 		}
 		if (!view_item->image)
 		{
-			view_item->image = item->image;
+			view_item->image = di_image(item);
 			if (view_item->image)
 				g_object_ref(view_item->image);
 		}
@@ -225,7 +225,7 @@ static DisplayStyle get_style(GtkCellRenderer *cell)
 
 	if (size == AUTO_SIZE_ICONS)
 	{
-		if (!view_item->image || view_item->image == item->image)
+		if (!view_item->image || view_item->image == di_image(item))
 			size = SMALL_ICONS;
 		else
 			size = HUGE_ICONS;
@@ -312,11 +312,13 @@ static void cell_icon_render(GtkCellRenderer    *cell,
 	DirItem *item;
 	DisplayStyle size;
 	gboolean selected = (flags & GTK_CELL_RENDERER_SELECTED) != 0;
+	GdkColor *color;
 	
 	g_return_if_fail(view_item != NULL);
 
 	item = view_item->item;
 	size = get_style(cell);
+	color = &widget->style->base[icon->view_details->filer_window->selection_state];
 
 	/* Draw the icon */
 
@@ -331,19 +333,19 @@ static void cell_icon_render(GtkCellRenderer    *cell,
 			area.width = MIN(area.width, SMALL_WIDTH);
 			area.x = cell_area->x + cell_area->width - area.width;
 			draw_small_icon(window, &area, item,
-					view_item->image, selected);
+					view_item->image, selected, color);
 
 			break;
 		}
 		case LARGE_ICONS:
 			draw_large_icon(window, cell_area, item,
-					view_item->image, selected);
+					view_item->image, selected, color);
 			break;
 		case HUGE_ICONS:
-			if (!item->image->huge_pixbuf)
-				pixmap_make_huge(item->image);
+			if (!di_image(item)->huge_pixbuf)
+				pixmap_make_huge(di_image(item));
 			draw_huge_icon(window, cell_area, item,
-					view_item->image, selected);
+					view_item->image, selected, color);
 			break;
 		default:
 			g_warning("Unknown size %d\n", size);
