@@ -1,7 +1,7 @@
 /*
- * $Id: diritem.c,v 1.35 2004/06/04 14:57:17 tal197 Exp $
+ * $Id: diritem.c,v 1.49 2005/11/12 17:12:34 tal197 Exp $
  *
- * Copyright (C) 2003, the ROX-Filer team.
+ * Copyright (C) 2005, the ROX-Filer team.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -46,8 +46,6 @@
 #include "pixmaps.h"
 #include "xtypes.h"
 
-static Option o_ignore_exec;
-
 #define RECENT_DELAY (5 * 60)	/* Time in seconds to consider a file recent */
 #define ABOUT_NOW(time) (diritem_recent_time - time < RECENT_DELAY)
 /* If you want to make use of the RECENT flag, make sure this is set to
@@ -56,7 +54,8 @@ static Option o_ignore_exec;
 time_t diritem_recent_time;
 
 /* Static prototypes */
-static void examine_dir(const guchar *path, DirItem *item, uid_t uid);
+static void examine_dir(const guchar *path, DirItem *item,
+			struct stat *link_target);
 
 /****************************************************************
  *			EXTERNAL INTERFACE			*
@@ -64,8 +63,6 @@ static void examine_dir(const guchar *path, DirItem *item, uid_t uid);
 
 void diritem_init(void)
 {
-	option_add_int(&o_ignore_exec, "display_ignore_exec", TRUE);
-
 	read_globicons();
 }
 
@@ -76,10 +73,10 @@ void diritem_restat(const guchar *path, DirItem *item, struct stat *parent)
 {
 	struct stat	info;
 
-	if (item->image)
+	if (item->_image)
 	{
-		g_object_unref(item->image);
-		item->image = NULL;
+		g_object_unref(item->_image);
+		item->_image = NULL;
 	}
 	item->flags = 0;
 	item->mime_type = NULL;
@@ -109,6 +106,9 @@ void diritem_restat(const guchar *path, DirItem *item, struct stat *parent)
 		if (ABOUT_NOW(item->mtime) || ABOUT_NOW(item->ctime))
 			item->flags |= ITEM_FLAG_RECENT;
 
+		if (xattr_have(path))
+			item->flags |= ITEM_FLAG_HAS_XATTR;
+
 		if (S_ISLNK(info.st_mode))
 		{
 			if (mc_stat(path, &info))
@@ -150,64 +150,64 @@ void diritem_restat(const guchar *path, DirItem *item, struct stat *parent)
 		 * of the *symlink*, but we really want the uid of the dir
 		 * to which the symlink points.
 		 */
-		examine_dir(path, item, info.st_uid);
+		examine_dir(path, item, &info);
 	}
 	else if (item->base_type == TYPE_FILE)
 	{
-		/* Type determined from path before checking for executables
-		 * because some mounts show everything as executable and we
-		 * still want to use the file name to set the mime type.
-		 */
-		if (item->flags & ITEM_FLAG_SYMLINK)
+		if (item->size == 0)
+			item->mime_type = text_plain;
+		else if (item->flags & ITEM_FLAG_SYMLINK)
 		{
 			guchar *link_path;
-			link_path = readlink_dup(path);
+			link_path = pathdup(path);
 			item->mime_type = type_from_path(link_path
-								? link_path
-								: path);
+					? link_path
+					: path);
 			g_free(link_path);
 		}
 		else
-			item->mime_type = xtype_get(path);
-
+			item->mime_type = type_from_path(path);
+	
 		/* Note: for symlinks we need the mode of the target */
 		if (info.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH))
 		{
 			/* Note that the flag is set for ALL executable
-			 * files, but the mime_type is only
-			 * application_executable if the file doesn't have a
-			 * known extension when that option is in force.
+			 * files, but the mime_type must also be executable
+			 * for clicking on the file to run it.
 			 */
 			item->flags |= ITEM_FLAG_EXEC_FILE;
 
-			if (!o_ignore_exec.int_value)
+			if (item->mime_type == NULL ||
+			    item->mime_type == application_octet_stream)
+			{
 				item->mime_type = application_executable;
+			}
+			else if (item->mime_type == text_plain &&
+			         !strchr(item->leafname, '.'))
+			{
+				item->mime_type = application_x_shellscript;
+			}
+		}		
+		else if (item->mime_type == application_x_desktop)
+		{
+			item->flags |= ITEM_FLAG_EXEC_FILE;
 		}
 
 		if (!item->mime_type)
-			item->mime_type = item->flags & ITEM_FLAG_EXEC_FILE
-						? application_executable
-						: text_plain;
+			item->mime_type = text_plain;
 
 		check_globicon(path, item);
+
+		if (item->mime_type == application_x_desktop && item->_image == NULL)
+		{
+			item->_image = g_fscache_lookup(desktop_icon_cache, path);
+		}
 	}
 	else
 		check_globicon(path, item);
 
-
 	if (!item->mime_type)
 		item->mime_type = mime_type_from_base_type(item->base_type);
-
-	if (!item->image)
-	{
-		if (item->base_type == TYPE_ERROR)
-		{
-			item->image = im_error;
-			g_object_ref(im_error);
-		}
-		else
-			item->image = type_to_icon(item->mime_type);
-	}
 }
 
 DirItem *diritem_new(const guchar *leafname)
@@ -217,9 +217,9 @@ DirItem *diritem_new(const guchar *leafname)
 	item = g_new(DirItem, 1);
 	item->leafname = g_strdup(leafname);
 	item->may_delete = FALSE;
-	item->image = NULL;
+	item->_image = NULL;
 	item->base_type = TYPE_UNKNOWN;
-	item->flags = 0;
+	item->flags = ITEM_FLAG_NEED_RESCAN_QUEUE;
 	item->mime_type = NULL;
 	item->leafname_collate = collate_key_new(leafname);
 
@@ -230,14 +230,27 @@ void diritem_free(DirItem *item)
 {
 	g_return_if_fail(item != NULL);
 
-	if (item->image)
-		g_object_unref(item->image);
-	item->image = NULL;
+	if (item->_image)
+		g_object_unref(item->_image);
+	item->_image = NULL;
 	collate_key_free(item->leafname_collate);
 	g_free(item->leafname);
 	g_free(item);
 }
 
+/* For use by di_image() only. Sets item->_image. */
+void _diritem_get_image(DirItem *item)
+{
+	g_return_if_fail(item->_image == NULL);
+
+	if (item->base_type == TYPE_ERROR)
+	{
+		item->_image = im_error;
+		g_object_ref(im_error);
+	}
+	else
+		item->_image = type_to_icon(item->mime_type);
+}
 
 /****************************************************************
  *			INTERNAL FUNCTIONS			*
@@ -247,14 +260,15 @@ void diritem_free(DirItem *item)
  * - Looks for an image (but maybe still NULL on error)
  * - Updates ITEM_FLAG_APPDIR
  *
- * KRJW (26 Jan 2003): Pass extra uid var, which will be the uid of
- * the dir given by `path' (or the uid of the dir pointed to by `path'
- * if `path' is a symlink).
+ * link_target contains stat info for the link target for symlinks (or for the
+ * item itself if not a link).
  */
-static void examine_dir(const guchar *path, DirItem *item, uid_t uid)
+static void examine_dir(const guchar *path, DirItem *item,
+			struct stat *link_target)
 {
 	struct stat info;
 	static GString *tmp = NULL;
+	uid_t uid = link_target->st_uid;
 
 	if (!tmp)
 		tmp = g_string_new(NULL);
@@ -267,6 +281,9 @@ static void examine_dir(const guchar *path, DirItem *item, uid_t uid)
 		return;		/* Try to avoid automounter problems */
 	}
 
+	if (link_target->st_mode & S_IWOTH)
+		return;		/* Don't trust world-writable dirs */
+
 	/* Finding the icon:
 	 *
 	 * - If it contains a .DirIcon then that's the icon
@@ -281,7 +298,7 @@ static void examine_dir(const guchar *path, DirItem *item, uid_t uid)
 
 	g_string_printf(tmp, "%s/.DirIcon", path);
 
-	if (item->image)
+	if (item->_image)
 		goto no_diricon;	/* Already got an icon */
 
 	if (mc_lstat(tmp->str, &info) != 0 || info.st_uid != uid)
@@ -294,7 +311,7 @@ static void examine_dir(const guchar *path, DirItem *item, uid_t uid)
 		goto no_diricon;	/* Too big, or non-regular file */
 
 	/* Try to load image; may still get NULL... */
-	item->image = g_fscache_lookup(pixmap_cache, tmp->str);
+	item->_image = g_fscache_lookup(pixmap_cache, tmp->str);
 
 no_diricon:
 
@@ -312,7 +329,7 @@ no_diricon:
 
 	/* Try to load AppIcon.xpm... */
 
-	if (item->image)
+	if (item->_image)
 		goto out;	/* Already got an icon */
 
 	g_string_truncate(tmp, tmp->len - 3);
@@ -329,14 +346,14 @@ no_diricon:
 		goto out;	/* Too big, or non-regular file */
 
 	/* Try to load image; may still get NULL... */
-	item->image = g_fscache_lookup(pixmap_cache, tmp->str);
+	item->_image = g_fscache_lookup(pixmap_cache, tmp->str);
 
 out:
 
-	if ((item->flags & ITEM_FLAG_APPDIR) && !item->image)
+	if ((item->flags & ITEM_FLAG_APPDIR) && !item->_image)
 	{
 		/* This is an application without an icon */
-		item->image = im_appdir;
-		g_object_ref(item->image);
+		item->_image = im_appdir;
+		g_object_ref(item->_image);
 	}
 }
