@@ -1,8 +1,8 @@
 /*
- * $Id: filer.c,v 1.348 2004/07/30 17:08:39 tal197 Exp $
+ * $Id: filer.c,v 1.372 2005/12/15 18:39:54 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2003, the ROX-Filer team.
+ * Copyright (C) 2005, the ROX-Filer team.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -156,6 +156,7 @@ static gboolean drag_motion(GtkWidget		*widget,
 static void load_settings(void);
 static void save_settings(void);
 static void check_settings(FilerWindow *filer_window);
+static char *tip_from_desktop_file(const char *full_path);
 
 static GdkCursor *busy_cursor = NULL;
 static GdkCursor *crosshair = NULL;
@@ -243,7 +244,6 @@ static gboolean if_deleted(gpointer item, gpointer removed)
  */
 void filer_window_set_size(FilerWindow *filer_window, int w, int h)
 {
-	GdkEvent *event;
 	GtkWidget *window;
 
 	g_return_if_fail(filer_window != NULL);
@@ -263,6 +263,7 @@ void filer_window_set_size(FilerWindow *filer_window, int w, int h)
 		gint x, y, m;
 		GtkRequisition	*req = &window->requisition;
 		GdkWindow *gdk_window = window->window;
+		GdkEvent *event;
 
 		w = MAX(req->width, w);
 		h = MAX(req->height, h);
@@ -292,21 +293,39 @@ void filer_window_set_size(FilerWindow *filer_window, int w, int h)
 		}
 		else
 			gdk_window_resize(gdk_window, w, h);
+
+		/* If the resize was triggered by a key press, keep
+		 * the pointer inside the window so that it doesn't
+		 * lose focus when using pointer-follows-mouse.
+		 */
+		event = gtk_get_current_event();
+		if (event && event->type == GDK_KEY_PRESS)
+		{
+			int x, y;
+			int nx, ny;
+
+			GdkWindow *win = filer_window->window->window;
+
+			gdk_window_get_pointer(filer_window->window->window,
+						&x, &y, NULL);
+
+			nx = CLAMP(x, 4, w - 4);
+			ny = CLAMP(y, 4, h - 4);
+			
+			if (nx != x || ny != y)
+			{
+				XWarpPointer(gdk_x11_drawable_get_xdisplay(win),
+						None,
+						gdk_x11_drawable_get_xid(win),
+						0, 0, 0, 0,
+						nx, ny);
+			}
+		}
+		if (event)
+			gdk_event_free(event);
 	}
 	else
 		gtk_window_set_default_size(GTK_WINDOW(window), w, h);
-
-	event = gtk_get_current_event();
-	if (event && event->type == GDK_KEY_PRESS)
-	{
-		GdkWindow *win = filer_window->window->window;
-		
-		XWarpPointer(gdk_x11_drawable_get_xdisplay(win),
-			     None,
-			     gdk_x11_drawable_get_xid(win),
-			     0, 0, 0, 0,
-			     w - 4, h - 4);
-	}
 }
 
 /* Called on a timeout while scanning or when scanning ends
@@ -314,6 +333,15 @@ void filer_window_set_size(FilerWindow *filer_window, int w, int h)
  */
 static gint open_filer_window(FilerWindow *filer_window)
 {
+	Settings *dir_settings;
+	gboolean force_resize;
+	
+	dir_settings = (Settings *) g_hash_table_lookup(settings_table,
+					      filer_window->sym_path);
+
+	force_resize = !(o_filer_auto_resize.int_value == RESIZE_NEVER &&
+			 dir_settings && dir_settings->flags & SET_POSITION);
+
 	view_style_changed(filer_window->view, 0);
 
 	if (filer_window->open_timeout)
@@ -324,13 +352,29 @@ static gint open_filer_window(FilerWindow *filer_window)
 
 	if (!GTK_WIDGET_VISIBLE(filer_window->window))
 	{
-		display_set_actual_size(filer_window, TRUE);
+		display_set_actual_size(filer_window, force_resize);
 		gtk_widget_show(filer_window->window);
 	}
 
 	return FALSE;
 }
 
+/* Look through all items we want to display, and queue a recheck on any
+ * that require it.
+ */
+static void queue_interesting(FilerWindow *filer_window)
+{
+	DirItem	*item;
+	ViewIter iter;
+
+	view_get_iter(filer_window->view, &iter, 0);
+	while ((item = iter.next(&iter)))
+	{
+		if (item->flags & ITEM_FLAG_NEED_RESCAN_QUEUE)
+			dir_queue_recheck(filer_window->directory, item);
+	}
+}
+
 static void update_display(Directory *dir,
 			DirAction	action,
 			GPtrArray	*items,
@@ -388,6 +432,9 @@ static void update_display(Directory *dir,
 		case DIR_ERROR_CHANGED:
 			filer_set_title(filer_window);
 			break;
+		case DIR_QUEUE_INTERESTING:
+			queue_interesting(filer_window);
+			break;
 	}
 }
 
@@ -424,6 +471,121 @@ static void detach(FilerWindow *filer_window)
 	filer_window->directory = NULL;
 }
 
+/* If 'start' was mounted by ROX-Filer, return it. Otherwise, try the
+ * parents up the tree.
+ * NULL if we're not in a user mount point.
+ * g_free() the result.
+ */
+static char *get_ancestor_user_mount_point(const char *start)
+{
+	char *path;
+
+	path = strdup(start);
+
+	while (1)
+	{
+		char *slash;
+
+		if (mount_is_user_mounted(path))
+			return path;
+
+		if (!path[1])
+		{
+			g_free(path);
+			return NULL;
+		}
+
+		slash = strrchr(path + 1, '/');
+		if (!slash)
+			slash = path + 1;
+		*slash = '\0';
+	}
+}
+
+static void umount_dialog_response(GtkWidget *dialog, int response, char *mount)
+{
+	if (response == GTK_RESPONSE_OK)
+	{
+		GList *list; 
+
+		list = g_list_prepend(NULL, mount);
+		action_mount(list, FALSE, FALSE, TRUE);
+		g_list_free(list);
+	}
+
+	g_free(mount);
+
+	gtk_widget_destroy(dialog);
+
+	one_less_window();
+}
+
+/* 'filer_window' shows a directory under 'mount'. If no other window also
+ * shows a directory under it, display a non-modal dialog offering to
+ * unmount the directory.
+ * 'mount' is freed by this function, either directly, or after the dialog
+ * closes.
+ */
+static void may_offer_unmount(FilerWindow *filer_window, char *mount)
+{
+	GtkWidget *dialog, *button;
+	GList	*next;
+	int len;
+	
+	len = strlen(mount);
+
+	for (next = all_filer_windows; next; next = next->next)
+	{
+		FilerWindow *other = (FilerWindow *) next->data;
+
+		if (other == filer_window)
+			continue;
+
+		if (strncmp(filer_window->real_path, other->real_path,
+			    len) != 0)
+			continue;
+
+		g_return_if_fail(
+			filer_window->real_path[len] != '/' ||
+			filer_window->real_path[len] != '\0');
+
+		if (other->real_path[len] != '/' &&
+		    other->real_path[len] != '\0')
+			continue;
+
+		/* Found another window. Don't offer to unmount. */
+		g_free(mount);
+		return;
+	}
+
+	dialog = gtk_message_dialog_new(NULL, 0, GTK_MESSAGE_QUESTION,
+			GTK_BUTTONS_NONE, 
+			_("Do you want to unmount this device?\n\n"
+			"Unmounting a device makes it safe to remove "
+			"the disk."), mount);
+
+	button = button_new_mixed(ROX_STOCK_MOUNTED, _("No change"));
+	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
+	gtk_dialog_add_action_widget(GTK_DIALOG(dialog), button,
+					GTK_RESPONSE_CANCEL);
+	gtk_widget_show(button);
+
+	button = button_new_mixed(ROX_STOCK_MOUNT, _("Unmount"));
+	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
+	gtk_dialog_add_action_widget(GTK_DIALOG(dialog), button,
+					GTK_RESPONSE_OK);
+	gtk_widget_show(button);
+
+	g_signal_connect(G_OBJECT(dialog), "response",
+			G_CALLBACK(umount_dialog_response), mount);
+
+	gtk_dialog_set_default_response(GTK_DIALOG(dialog),
+			GTK_RESPONSE_OK);
+
+	number_of_windows++;
+	gtk_widget_show(dialog);
+}
+
 /* Returns TRUE to prevent closing the window. May offer to unmount a
  * device.
  */
@@ -431,32 +593,12 @@ gboolean filer_window_delete(GtkWidget *window,
 			     GdkEvent *unused, /* (may be NULL) */
 			     FilerWindow *filer_window)
 {
-	if (mount_is_user_mounted(filer_window->real_path))
-	{
-		int action;
+	char *mount;
 
-		action = get_choice(PROJECT,
-			_("Do you want to unmount this device?\n\n"
-			"Unmounting a device makes it safe to remove "
-			"the disk."), 3,
-			GTK_STOCK_CANCEL, NULL,
-			GTK_STOCK_CLOSE, NULL,
-			ROX_STOCK_MOUNT, _("Unmount"));
+	mount = get_ancestor_user_mount_point(filer_window->real_path);
 
-		if (action == 0)
-			return TRUE;	/* Cancel close operation */
-
-		if (action == 2)
-		{
-			GList *list; 
-
-			list = g_list_prepend(NULL, filer_window->sym_path);
-			action_mount(list, FALSE, TRUE);
-			g_list_free(list);
-
-			return TRUE;
-		}
-	}
+	if (mount)
+		may_offer_unmount(filer_window, mount);
 
 	return FALSE;
 }
@@ -604,12 +746,12 @@ static void selection_get(GtkWidget *widget,
 	}
 	
 	if (reply->len > 0)
-		gtk_selection_data_set(selection_data, xa_string,
-				8, reply->str + 1, reply->len - 1);
+		gtk_selection_data_set_text(selection_data,
+				reply->str + 1, reply->len - 1);
 	else
 	{
 		g_warning("Attempt to paste empty selection!");
-		gtk_selection_data_set(selection_data, xa_string, 8, "", 0);
+		gtk_selection_data_set_text(selection_data, "", 0);
 	}
 
 	g_string_free(reply, TRUE);
@@ -623,6 +765,8 @@ static void selection_get(GtkWidget *widget,
  */
 void filer_selection_changed(FilerWindow *filer_window, gint time)
 {
+	g_return_if_fail(filer_window != NULL);
+
 	toolbar_update_info(filer_window);
 
 	if (window_with_primary == filer_window)
@@ -666,7 +810,7 @@ void filer_openitem(FilerWindow *filer_window, ViewIter *iter, OpenFlags flags)
 		return;
 	}
 
-	if (!item->image)
+	if (item->base_type == TYPE_UNKNOWN)
 		dir_update_item(filer_window->directory, item->leafname);
 
 	if (item->base_type == TYPE_DIRECTORY)
@@ -789,7 +933,7 @@ static xmlNode *group_find(char *name)
 	gchar *path;
 
 	/* Update the groups, if possible */
-	path = choices_find_path_load("Groups.xml", PROJECT);
+	path = choices_find_xdg_path_load("Groups.xml", PROJECT, SITE);
 	if (path)
 	{
 		XMLwrapper *wrapper;
@@ -860,7 +1004,8 @@ static void group_save(FilerWindow *filer_window, char *name)
 	while ((item = iter.next(&iter)))
 		xmlNewTextChild(group, NULL, "item", item->leafname);
 
-	save_path = choices_find_path_save("Groups.xml", PROJECT, TRUE);
+	save_path = choices_find_xdg_path_save("Groups.xml", PROJECT, SITE,
+					       TRUE);
 	if (save_path)
 	{
 		save_xml_file(groups->doc, save_path);
@@ -929,10 +1074,14 @@ static void group_restore(FilerWindow *filer_window, char *name)
 static gboolean popup_menu(GtkWidget *widget, FilerWindow *filer_window)
 {
 	ViewIter iter;
+	GdkEvent *event;
 
 	view_get_cursor(filer_window->view, &iter);
 
-	show_filer_menu(filer_window, NULL, &iter);
+	event = gtk_get_current_event();
+	show_filer_menu(filer_window, event, &iter);
+	if (event)
+		gdk_event_free(event);
 
 	return TRUE;
 }
@@ -1069,6 +1218,10 @@ void change_to_parent(FilerWindow *filer_window)
 
 	if (current[0] == '/' && current[1] == '\0')
 		return;		/* Already in the root */
+
+	if (mount_is_user_mounted(filer_window->real_path))
+		may_offer_unmount(filer_window,
+				g_strdup(filer_window->real_path));
 	
 	dir = g_path_get_dirname(current);
 	filer_change_to(filer_window, dir, g_basename(current));
@@ -1216,6 +1369,7 @@ FilerWindow *filer_opendir(const char *path, FilerWindow *src_win,
 	SortType	s_type;
 	GtkSortType	s_order;
 	Settings	*dir_settings = NULL;
+	gboolean 	force_resize = TRUE;
 
 	/* Get the real pathname of the directory and copy it */
 	real_path = pathdup(path);
@@ -1320,11 +1474,13 @@ FilerWindow *filer_opendir(const char *path, FilerWindow *src_win,
 			filer_window->show_hidden = dir_settings->show_hidden;
 
 		if (dir_settings->flags & SET_STYLE)
-			filer_window->display_style =
-				dir_settings->display_style;
+			dstyle = dir_settings->display_style;
 
 		if (dir_settings->flags & SET_DETAILS)
-			filer_window->details_type = dir_settings->details_type;
+		{
+			dtype = dir_settings->details_type;
+			filer_window->view_type = dir_settings->view_type;
+		}
 
 		if (dir_settings->flags & SET_SORT)
 		{
@@ -1349,19 +1505,24 @@ FilerWindow *filer_opendir(const char *path, FilerWindow *src_win,
 
 	if (dir_settings)
 	{
-		if(dir_settings->flags & SET_POSITION) 
+		if (dir_settings->flags & SET_POSITION)
+		{
 			gtk_window_move(GTK_WINDOW(filer_window->window),
 					    dir_settings->x, dir_settings->y);
-		if(dir_settings->flags & SET_SIZE) 
+		}
+		if (dir_settings->flags & SET_SIZE)
+		{
 			filer_window_set_size(filer_window,
 					      dir_settings->width,
 					      dir_settings->height);
+			force_resize = o_filer_auto_resize.int_value != RESIZE_NEVER;
+		}
 	}
 
 	/* Connect to all the signal handlers */
 	filer_add_signals(filer_window);
 
-	display_set_layout(filer_window, dstyle, dtype, TRUE);
+	display_set_layout(filer_window, dstyle, dtype, force_resize);
 	display_set_sort_type(filer_window, s_type, s_order);
 
 	/* Open the window after a timeout, or when scanning stops.
@@ -1489,6 +1650,8 @@ static void filer_add_widgets(FilerWindow *filer_window, const gchar *wm_class)
 	/* Create the top-level window widget */
 	filer_window->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	filer_set_title(filer_window);
+	gtk_widget_set_name(filer_window->window, "rox-filer");
+
 	if (wm_class)
 		gtk_window_set_wmclass(GTK_WINDOW(filer_window->window),
 				       wm_class, PROJECT);
@@ -1575,9 +1738,9 @@ static void filer_add_signals(FilerWindow *filer_window)
 	GtkTargetEntry 	target_table[] =
 	{
 		{"text/uri-list", 0, TARGET_URI_LIST},
+		{"UTF8_STRING", 0, TARGET_STRING},
 		{"STRING", 0, TARGET_STRING},
 		{"COMPOUND_TEXT", 0, TARGET_STRING},/* XXX: Treats as STRING */
-		{"UTF8_STRING", 0, TARGET_STRING},
 	};
 
 	/* Events on the top-level window */
@@ -1658,7 +1821,13 @@ void filer_update_all(void)
 		 */
 		next = next->next;
 
-		filer_update_dir(filer_window, TRUE);
+		/* Don't trigger a refresh if we're already scanning.
+		 * Otherwise, two views of a single directory will trigger
+		 * two scans.
+		 */
+		if (filer_window->directory &&
+		    !filer_window->directory->scanning)
+			filer_update_dir(filer_window, TRUE);
 	}
 }
 
@@ -1814,7 +1983,6 @@ void filer_set_title(FilerWindow *filer_window)
 {
 	gchar	*title = NULL;
 	guchar	*flags = "";
-	gchar  *hidden = "";
 
 	if (filer_window->scanning ||
 	    filer_window->filter != FILER_SHOW_ALL ||
@@ -1822,13 +1990,13 @@ void filer_set_title(FilerWindow *filer_window)
 	{
 		if (o_short_flag_names.int_value)
 		{
+			const gchar  *hidden = "";
 
 			switch(filer_window->filter) {
 			case FILER_SHOW_ALL:
-				hidden=filer_window->show_hidden? "A": "";
+				hidden=filer_window->show_hidden? _("A") : "";
 				break;
-			case FILER_SHOW_GLOB:   hidden="G"; break;
-			case FILER_SHOW_REGEXP: hidden="R"; break;
+			case FILER_SHOW_GLOB:   hidden =  _("G"); break;
 			default: break;
 			}
 
@@ -1840,20 +2008,19 @@ void filer_set_title(FilerWindow *filer_window)
 		}
 		else
 		{
+			gchar  *hidden = NULL;
+
 			switch(filer_window->filter) {
 			case FILER_SHOW_ALL:
-				hidden=g_strdup(filer_window->show_hidden? "All, ": "");
+				hidden = g_strdup(filer_window->show_hidden
+						? _("All, ") : "");
 				break;
 			case FILER_SHOW_GLOB:
-				hidden=g_strdup_printf("Glob (%s), ",
-						 filer_window->filter_string);
-				break;
-			case FILER_SHOW_REGEXP:
-				hidden=g_strdup_printf("Regexp (%s), ",
+				hidden = g_strdup_printf(_("Glob (%s), "),
 						 filer_window->filter_string);
 				break;
 			default:
-				hidden=g_strdup("");
+				hidden  =g_strdup("");
 				break;
 			}
 			flags = g_strconcat(" (",
@@ -2040,6 +2207,12 @@ static void start_thumb_scanning(FilerWindow *filer_window)
 /* Set this image to be loaded some time in the future */
 void filer_create_thumb(FilerWindow *filer_window, const gchar *path)
 {
+	if (g_list_find_custom(filer_window->thumb_queue, path,
+			       (GCompareFunc) strcmp))
+		return;
+
+	if (!filer_window->thumb_queue)
+		filer_window->max_thumbs=0;
 	filer_window->max_thumbs++;
 
 	filer_window->thumb_queue = g_list_append(filer_window->thumb_queue,
@@ -2157,6 +2330,17 @@ void filer_add_tip_details(FilerWindow *filer_window,
 		if (info)
 			g_object_unref(info);
 	}
+	else if (item->mime_type == application_x_desktop)
+	{
+		char *summary;
+		summary = tip_from_desktop_file(fullpath);
+		if (summary)
+		{
+			g_string_append(tip, summary);
+			g_string_append_c(tip, '\n');
+			g_free(summary);
+		}
+	}
 
 	if (!g_utf8_validate(item->leafname, -1, NULL))
 		g_string_append(tip,
@@ -2186,11 +2370,12 @@ static guchar *filer_create_uri_list(FilerWindow *filer_window)
 	view_get_iter(filer_window->view, &iter, VIEW_ITER_SELECTED);
 	while ((item = iter.next(&iter)))
 	{
-		gchar *uri, *path;
+		EscapedPath *uri;
+		char *path;
 		
 		path = g_strconcat(leader->str, item->leafname, NULL);
 		uri = encode_path_as_uri(path);
-		g_string_append(string, uri);
+		g_string_append(string, (char *) uri);
 		g_string_append(string, "\r\n");
 		g_free(path);
 		g_free(uri);
@@ -2218,6 +2403,9 @@ void filer_perform_action(FilerWindow *filer_window, GdkEventButton *event)
 	view_get_iter_at_point(view, &iter, event->window, event->x, event->y);
 	item = iter.peek(&iter);
 
+	if (item && view_cursor_visible(view))
+		view_cursor_to_iter(view, &iter);
+
 	if (item && event->button == 1 &&
 		view_get_selected(view, &iter) &&
 		filer_window->selection_state == GTK_STATE_INSENSITIVE)
@@ -2461,7 +2649,7 @@ gint filer_motion_notify(FilerWindow *filer_window, GdkEventMotion *event)
 
 	if (view_count_selected(view) == 1)
 	{
-		if (!item->image)
+		if (item->base_type == TYPE_UNKNOWN)
 			item = dir_update_item(filer_window->directory,
 						item->leafname);
 
@@ -2473,7 +2661,7 @@ gint filer_motion_notify(FilerWindow *filer_window, GdkEventMotion *event)
 
 		drag_one_item(GTK_WIDGET(view), event,
 			make_path(filer_window->sym_path, item->leafname),
-			item, item->image);
+			item, di_image(item));
 #if 0
 		/* XXX: Use thumbnail */
 			item, view ? view->image : NULL);
@@ -2693,18 +2881,17 @@ void filer_refresh(FilerWindow *filer_window)
 
 gboolean filer_match_filter(FilerWindow *filer_window, const gchar *filename)
 {
+	g_return_val_if_fail(filename != NULL, FALSE);
+
 	if(filename[0]=='.' &&
 	   (!filer_window->temp_show_hidden && !filer_window->show_hidden))
 		return FALSE;
 
-	/*printf("%d %s\n", filer_window->filter, filename);*/
 	switch(filer_window->filter) {
 	case FILER_SHOW_GLOB:
 		return fnmatch(filer_window->filter_string,
 			       filename, 0)==0;
 		
-	case FILER_SHOW_REGEXP: /* Unimplemented */
-
 	case FILER_SHOW_ALL:
 	default:
 		break;
@@ -2718,55 +2905,55 @@ void filer_set_hidden(FilerWindow *filer_window, gboolean hidden)
 	filer_window->show_hidden=hidden;
 }
 
-void filer_set_filter(FilerWindow *filer_window, FilterType type,
+/* Set the filter type. Returns TRUE if the type has changed
+ * (must call filer_detach_rescan).
+ */
+gboolean filer_set_filter(FilerWindow *filer_window, FilterType type,
 			     const gchar *filter_string)
 {
-	/*printf("filer_set_filter(%p, %d, %s)\n", filer_window, type,
-	  filter_string? filter_string: "NULL");*/
 	/* Is this new filter the same as the old one? */
-	if(filer_window->filter==type) {
-		switch(filer_window->filter) {
+	if (filer_window->filter == type)
+	{
+		switch(filer_window->filter)
+		{
 		case FILER_SHOW_ALL:
-			return;
+			return FALSE;
 		case FILER_SHOW_GLOB:
-		case FILER_SHOW_REGEXP:
-			if(strcmp(filer_window->filter_string,
-				  filter_string)==0)
-				return;
+			if (strcmp(filer_window->filter_string,
+				   filter_string) == 0)
+				return FALSE;
 			break;
 		}
 	}
 
 	/* Clean up old filter */
-	if(filer_window->filter_string) {
+	if (filer_window->filter_string)
+	{
 		g_free(filer_window->filter_string);
-		filer_window->filter_string=NULL;
+		filer_window->filter_string = NULL;
 	}
 	/* Also clean up compiled regexp when implemented */
 
-	/*printf("set %d %s\n", type,
-	  filter_string? filter_string: "NULL");*/
-	filer_window->filter=type;
-	switch(type) {
+	filer_window->filter = type;
+
+	switch(type)
+	{
 	case FILER_SHOW_ALL:
 		/* No extra work */
 		break;
 
 	case FILER_SHOW_GLOB:
-		filer_window->filter_string=g_strdup(filter_string);
-		break;
-
-	case FILER_SHOW_REGEXP:
-		filer_window->filter_string=g_strdup(filter_string);
-		/* Compile the pattern */
+		filer_window->filter_string = g_strdup(filter_string);
 		break;
 
 	default:
 		/* oops */
-		filer_window->filter=FILER_SHOW_ALL;
-		report_error("Impossible: filter type %d", type);
+		filer_window->filter = FILER_SHOW_ALL;
+		g_warning("Impossible: filter type %d", type);
 		break;
 	}
+
+	return TRUE;
 }
 
 /* Setting stuff */
@@ -2790,22 +2977,15 @@ static void settings_free(Settings *set)
 	g_free(set);
 }
 
-static gboolean free_settings(gpointer key, gpointer value, gpointer data)
-{
-	if(!data || strcmp(data, key)==0)
-		settings_free(value);
-
-	return TRUE;
-}
-
 static void store_settings(Settings *set)
 {
 	Settings *old;
 
 	old=g_hash_table_lookup(settings_table, set->path);
-	if(old) {
-		g_hash_table_foreach_remove(settings_table, free_settings,
-					    set->path);
+	if(old)
+	{
+		g_hash_table_remove(settings_table, set->path);
+		settings_free(old);
 	}
 
 	g_hash_table_insert(settings_table, set->path, set);
@@ -2868,7 +3048,7 @@ static void load_settings(void)
 	gchar *path;
 	XMLwrapper *settings_doc=NULL;
 
-	path=choices_find_path_load("Settings.xml", "ROX-Filer");
+	path=choices_find_xdg_path_load("Settings.xml", PROJECT, SITE);
 	if(path) {
 		settings_doc=xml_new(path);
 		g_free(path);
@@ -2982,7 +3162,7 @@ static void save_settings(void)
 {
 	gchar *path;
 
-	path=choices_find_path_save("Settings.xml", "ROX-Filer", TRUE);
+	path=choices_find_xdg_path_save("Settings.xml", PROJECT, SITE, TRUE);
 	if(path) {
 		xmlDocPtr doc = xmlNewDoc("1.0");
 		xmlDocSetRootElement(doc, xmlNewDocNode(doc, NULL,
@@ -3208,3 +3388,24 @@ void filer_save_settings(FilerWindow *fwin)
 
 	gtk_widget_show_all(set_win->window);
 }
+
+static char *tip_from_desktop_file(const char *full_path)
+{
+	GError *error = NULL;
+	char *comment = NULL;
+
+	comment = get_value_from_desktop_file(full_path,
+			"Desktop Entry", "Comment", &error);
+	if (error)
+	{
+		delayed_error("Failed to parse .desktop file '%s':\n%s",
+				full_path, error->message);
+		goto err;
+	}
+
+err:
+	if (error != NULL)
+		g_error_free(error);
+
+	return comment;
+}
