@@ -36,6 +36,13 @@ typedef enum
 	VIEW_TYPE_DETAILS = 1		/* TreeView details list */
 } ViewType;
 
+/* Filter types */
+typedef enum
+{
+	FILER_SHOW_ALL,           /* Show all files, modified by show_hidden */
+	FILER_SHOW_GLOB,          /* Show files that match a glob pattern */
+} FilterType;
+
 /* iter's next method has just returned the clicked item... */
 typedef void (*TargetFunc)(FilerWindow *filer_window,
 			   ViewIter *iter,
@@ -73,6 +80,9 @@ struct _FilerWindow
 	int		mini_cursor_base;	/* XXX */
 	MiniType	mini_type;
 
+	FilterType      filter;
+	gchar           *filter_string;  /* Glob or regexp pattern */
+	gchar           *regexp;         /* Compiled regexp pattern */
 	/* TRUE if hidden files are shown because the minibuffer leafname
 	 * starts with a dot.
 	 */
@@ -95,6 +105,8 @@ struct _FilerWindow
 	int		max_thumbs;		/* total for this batch */
 
 	gint		auto_scroll;		/* Timer */
+
+	char		*window_id;		/* For remote control */
 };
 
 extern FilerWindow 	*window_with_focus;
@@ -117,13 +129,14 @@ void filer_close_recursive(const char *path);
 void filer_change_to(FilerWindow *filer_window,
 			const char *path, const char *from);
 gboolean filer_exists(FilerWindow *filer_window);
+FilerWindow *filer_get_by_id(const char *id);
+void filer_set_id(FilerWindow *, const char *id);
 void filer_open_parent(FilerWindow *filer_window);
 void filer_detach_rescan(FilerWindow *filer_window);
 void filer_target_mode(FilerWindow	*filer_window,
 			TargetFunc	fn,
 			gpointer	data,
 			const char	*reason);
-void filer_window_autosize(FilerWindow *filer_window);
 GList *filer_selected_items(FilerWindow *filer_window);
 void filer_create_thumb(FilerWindow *filer_window, const gchar *pathname);
 void filer_cancel_thumbnails(FilerWindow *filer_window);
@@ -144,5 +157,13 @@ gint filer_motion_notify(FilerWindow *filer_window, GdkEventMotion *event);
 gint filer_key_press_event(GtkWidget *widget, GdkEventKey *event,
 			   FilerWindow *filer_window);
 void filer_set_autoscroll(FilerWindow *filer_window, gboolean auto_scroll);
+void filer_refresh(FilerWindow *filer_window);
+
+gboolean filer_match_filter(FilerWindow *filer_window, const gchar *filename);
+gboolean filer_set_filter(FilerWindow *filer_window,
+			  FilterType type, const gchar *filter_string);
+void filer_set_hidden(FilerWindow *fwin, gboolean hidden);
+void filer_next_selected(FilerWindow *filer_window, int dir);
+void filer_save_settings(FilerWindow *fwin);
 
 #endif /* _FILER_H */
