@@ -1,8 +1,8 @@
 /*
- * $Id: gui_support.c,v 1.117 2004/07/30 17:08:39 tal197 Exp $
+ * $Id: gui_support.c,v 1.137 2005/12/16 20:57:53 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2003, the ROX-Filer team.
+ * Copyright (C) 2005, the ROX-Filer team.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -117,9 +117,6 @@ void gui_support_init()
 	
 	xa_cardinal = gdk_atom_intern("CARDINAL", FALSE);
 
-	/* This call starts returning strange values after a while, so get
-	 * the result here during init.
-	 */
 	gui_store_screen_geometry(gdk_screen_get_default());
 
 	/* Work around the scrollbar placement bug */
@@ -217,7 +214,7 @@ void report_error(const char *message, ...)
 	run_error_info_dialog(GTK_MESSAGE_ERROR, message, args);
 }
 
-void set_cardinal_property(GdkWindow *window, GdkAtom prop, guint32 value)
+void set_cardinal_property(GdkWindow *window, GdkAtom prop, gulong value)
 {
 	gdk_property_change(window, prop, xa_cardinal, 32,
 				GDK_PROP_MODE_REPLACE, (gchar *) &value, 1);
@@ -229,11 +226,10 @@ void set_cardinal_property(GdkWindow *window, GdkAtom prop, guint32 value)
 void make_panel_window(GtkWidget *widget)
 {
 	static gboolean need_init = TRUE;
-	static GdkAtom xa_state, xa_atom, xa_net_state, xa_hints, xa_win_hints;
+	static GdkAtom xa_state, xa_atom, xa_hints, xa_win_hints;
 	static GdkAtom xa_NET_WM_DESKTOP;
-	static GdkAtom state_list[3];
 	GdkWindow *window = widget->window;
-	gint32  wm_hints_values[] = {1, False, 0, 0, 0, 0, 0, 0};
+	long wm_hints_values[] = {1, False, 0, 0, 0, 0, 0, 0};
 	GdkAtom	wm_protocols[2];
 
 	g_return_if_fail(window != NULL);
@@ -249,19 +245,9 @@ void make_panel_window(GtkWidget *widget)
 		xa_win_hints = gdk_atom_intern("_WIN_HINTS", FALSE);
 		xa_state = gdk_atom_intern("_WIN_STATE", FALSE);
 		xa_atom = gdk_atom_intern("ATOM", FALSE);
-		xa_net_state = gdk_atom_intern("_NET_WM_STATE", FALSE);
 		xa_hints = gdk_atom_intern("WM_HINTS", FALSE);
 		xa_NET_WM_DESKTOP = gdk_atom_intern("_NET_WM_DESKTOP", FALSE);
 
-		/* Note: Starting with Gtk+-1.3.12, Gtk+ converts GdkAtoms
-		 * to X atoms automatically when the type is ATOM.
-		 */
-		state_list[0] = gdk_atom_intern("_NET_WM_STATE_STICKY", FALSE);
-		state_list[1] = gdk_atom_intern("_NET_WM_STATE_SKIP_PAGER",
-						FALSE);
-		state_list[2] = gdk_atom_intern("_NET_WM_STATE_SKIP_TASKBAR",
-						FALSE);
-		
 		need_init = FALSE;
 	}
 	
@@ -269,10 +255,6 @@ void make_panel_window(GtkWidget *widget)
 	gdk_window_set_functions(window, 0);
 	gtk_window_set_resizable(GTK_WINDOW(widget), FALSE);
 
-	/* Note: DON'T do gtk_window_stick(). Setting the state via
-	 * gdk will override our other atoms (pager/taskbar).
-	 */
-
 	/* Don't hide panel/pinboard windows initially (WIN_STATE_HIDDEN).
 	 * Needed for IceWM - Christopher Arndt <chris.arndt@web.de>
 	 */
@@ -287,12 +269,9 @@ void make_panel_window(GtkWidget *widget)
 	/* Appear on all workspaces */
 	set_cardinal_property(window, xa_NET_WM_DESKTOP, 0xffffffff);
 
-	gdk_property_change(window, xa_net_state, xa_atom, 32,
-			GDK_PROP_MODE_APPEND, (guchar *) state_list, 3);
-
 	gdk_property_change(window, xa_hints, xa_hints, 32,
 			GDK_PROP_MODE_REPLACE, (guchar *) wm_hints_values,
-			sizeof(wm_hints_values) / sizeof(gint32));
+			sizeof(wm_hints_values) / sizeof(long));
 
 	wm_protocols[0] = gdk_atom_intern("WM_DELETE_WINDOW", FALSE);
 	wm_protocols[1] = gdk_atom_intern("_NET_WM_PING", FALSE);
@@ -300,6 +279,20 @@ void make_panel_window(GtkWidget *widget)
 			gdk_atom_intern("WM_PROTOCOLS", FALSE), xa_atom, 32,
 			GDK_PROP_MODE_REPLACE, (guchar *) wm_protocols,
 			sizeof(wm_protocols) / sizeof(GdkAtom));
+
+	gdk_window_set_skip_taskbar_hint(window, TRUE);
+	gdk_window_set_skip_pager_hint(window, TRUE);
+
+	if (g_object_class_find_property(G_OBJECT_GET_CLASS(widget),
+					"accept_focus"))
+	{
+		GValue vfalse = { 0, };
+		g_value_init(&vfalse, G_TYPE_BOOLEAN);
+		g_value_set_boolean(&vfalse, FALSE);
+		g_object_set_property(G_OBJECT(widget),
+					"accept_focus", &vfalse);
+		g_value_unset(&vfalse);
+	}
 }
 
 static gboolean error_idle_cb(gpointer data)
@@ -375,7 +368,7 @@ GtkWidget *new_help_button(HelpFunc show_help, gpointer data)
 	
 	b = gtk_button_new();
 	gtk_button_set_relief(GTK_BUTTON(b), GTK_RELIEF_NONE);
-	icon = gtk_image_new_from_stock(GTK_STOCK_DIALOG_INFO,
+	icon = gtk_image_new_from_stock(GTK_STOCK_HELP,
 					GTK_ICON_SIZE_SMALL_TOOLBAR);
 	gtk_container_add(GTK_CONTAINER(b), icon);
 	g_signal_connect_swapped(b, "clicked", G_CALLBACK(show_help), data);
@@ -424,7 +417,8 @@ void parse_file(const char *path, ParseFunc *parse_line)
 		_("Error in '%s' file at line %d: "
 		"\n\"%s\"\n"
 		"This may be due to upgrading from a previous version of "
-		"ROX-Filer. Open the Options window and click on Save.\n"
+		"ROX-Filer. Open the Options window and try changing something "
+		"and then changing it back (causing the file to be resaved).\n"
 		"Further errors will be ignored."),
 					path,
 					line_number,
@@ -620,17 +614,11 @@ GtkWidget *button_new_mixed(const char *stock, const char *message)
 /* Highlight entry in red if 'error' is TRUE */
 void entry_set_error(GtkWidget *entry, gboolean error)
 {
-	GdkColor red = {0, 0xffff, 0, 0};
-	static gboolean need_init = TRUE;
-	static GdkColor normal;
+	const GdkColor red = {0, 0xffff, 0, 0};
+	const GdkColor white = {0, 0xffff, 0xffff, 0xffff};
 
-	if (need_init)
-	{
-		normal = entry->style->text[GTK_STATE_NORMAL];
-		need_init = FALSE;
-	}
-
-	gtk_widget_modify_text(entry, GTK_STATE_NORMAL, error ? &red : &normal);
+	gtk_widget_modify_text(entry, GTK_STATE_NORMAL, error ? &red : NULL);
+	gtk_widget_modify_base(entry, GTK_STATE_NORMAL, error ? &white : NULL);
 }
 
 /* Change stacking position of higher to be just above lower.
@@ -853,7 +841,7 @@ gboolean confirm(const gchar *message, const gchar *stock, const gchar *action)
 struct _Radios {
 	GList *widgets;
 
-	void (*changed)(gpointer data);
+	void (*changed)(Radios *, gpointer data);
 	gpointer changed_data;
 };
 
@@ -863,7 +851,7 @@ struct _Radios {
  * changed(data) is called (if not NULL) when pack is called, and on any
  * change after that.
  */
-Radios *radios_new(void (*changed)(gpointer data), gpointer data)
+Radios *radios_new(void (*changed)(Radios *, gpointer data), gpointer data)
 {
 	Radios *radios;
 
@@ -923,8 +911,11 @@ static void radio_toggled(GtkToggleButton *button, Radios *radios)
 {
 	g_return_if_fail(radios != NULL);
 
+	if (button && !gtk_toggle_button_get_active(button))
+		return;	/* Stop double-notifies */
+
 	if (radios->changed)
-		radios->changed(radios->changed_data);
+		radios->changed(radios, radios->changed_data);
 }
 
 void radios_pack(Radios *radios, GtkBox *box)
@@ -988,7 +979,7 @@ gint radios_get_value(Radios *radios)
 	return -1;
 }
 
-/* Convert a list of URIs as a string into a GList of URIs.
+/* Convert a list of URIs as a string into a GList of EscapedPath URIs.
  * No unescaping is done.
  * Lines beginning with # are skipped.
  * The text block passed in is zero terminated (after the final CRLF)
@@ -1190,3 +1181,386 @@ static void gui_get_monitor_adjacent(int monitor, MonitorAdjacent *adj)
 		}
 	}
 }
+
+static void rox_wmspec_change_state(gboolean add, GdkWindow *window,
+				    GdkAtom state1, GdkAtom state2)
+{
+	GdkDisplay *display = gdk_drawable_get_display(GDK_DRAWABLE(window));
+	XEvent xev;
+	
+#define _NET_WM_STATE_REMOVE        0    /* remove/unset property */
+#define _NET_WM_STATE_ADD           1    /* add/set property */
+#define _NET_WM_STATE_TOGGLE        2    /* toggle property  */  
+
+	xev.xclient.type = ClientMessage;
+	xev.xclient.serial = 0;
+	xev.xclient.send_event = True;
+	xev.xclient.window = GDK_WINDOW_XID(window);
+	xev.xclient.message_type = gdk_x11_get_xatom_by_name_for_display(
+			display, "_NET_WM_STATE");
+	xev.xclient.format = 32;
+	xev.xclient.data.l[0] = add ? _NET_WM_STATE_ADD : _NET_WM_STATE_REMOVE;
+	xev.xclient.data.l[1] = gdk_x11_atom_to_xatom_for_display(display,
+			state1);
+	xev.xclient.data.l[2] = gdk_x11_atom_to_xatom_for_display(display,
+			state2);
+	xev.xclient.data.l[3] = 0;
+	xev.xclient.data.l[4] = 0;
+	
+	XSendEvent(GDK_DISPLAY_XDISPLAY(display),
+		   GDK_WINDOW_XID(
+			gdk_screen_get_root_window(
+				gdk_drawable_get_screen(GDK_DRAWABLE(window)))),
+		   False,
+		   SubstructureRedirectMask | SubstructureNotifyMask,
+		   &xev);
+}
+
+void keep_below(GdkWindow *window, gboolean setting)
+{
+	g_return_if_fail(GDK_IS_WINDOW(window));
+
+	if (GDK_WINDOW_DESTROYED(window))
+		return;
+
+	if (gdk_window_is_visible(window))
+	{
+		if (setting)
+		{
+			rox_wmspec_change_state(FALSE, window,
+				gdk_atom_intern("_NET_WM_STATE_ABOVE", FALSE),
+				GDK_NONE);
+		}
+		rox_wmspec_change_state(setting, window,
+				gdk_atom_intern("_NET_WM_STATE_BELOW", FALSE),
+				GDK_NONE);
+	}
+#if 0
+	else
+	{
+#if GTK_CHECK_VERSION(2,4,0)
+	  gdk_synthesize_window_state(window,
+				setting ? GDK_WINDOW_STATE_ABOVE :
+					GDK_WINDOW_STATE_BELOW,
+				setting ? GDK_WINDOW_STATE_BELOW : 0);
+#endif
+	}
+#endif
+}
+
+static void
+size_prepared_cb (GdkPixbufLoader *loader, 
+		  int              width,
+		  int              height,
+		  gpointer         data)
+{
+	struct {
+		gint width;
+		gint height;
+		gboolean preserve_aspect_ratio;
+	} *info = data;
+
+	g_return_if_fail (width > 0 && height > 0);
+
+	if(info->preserve_aspect_ratio) {
+		if ((double)height * (double)info->width >
+		    (double)width * (double)info->height) {
+			width = 0.5 + (double)width * (double)info->height / (double)height;
+			height = info->height;
+		} else {
+			height = 0.5 + (double)height * (double)info->width / (double)width;
+			width = info->width;
+		}
+	} else {
+		width = info->width;
+		height = info->height;
+	}
+	
+	gdk_pixbuf_loader_set_size (loader, width, height);
+}
+
+/**
+ * rox_pixbuf_new_from_file_at_scale:
+ * @filename: Name of file to load.
+ * @width: The width the image should have
+ * @height: The height the image should have
+ * @preserve_aspect_ratio: %TRUE to preserve the image's aspect ratio
+ * @error: Return location for an error
+ *
+ * Creates a new pixbuf by loading an image from a file.  The file format is
+ * detected automatically. If %NULL is returned, then @error will be set.
+ * Possible errors are in the #GDK_PIXBUF_ERROR and #G_FILE_ERROR domains.
+ * The image will be scaled to fit in the requested size, optionally preserving
+ * the image's aspect ratio.
+ *
+ * Return value: A newly-created pixbuf with a reference count of 1, or %NULL 
+ * if any of several error conditions occurred:  the file could not be opened,
+ * there was no loader for the file's format, there was not enough memory to
+ * allocate the image buffer, or the image file contained invalid data.
+ *
+ * Taken from GTK 2.6.
+ **/
+GdkPixbuf *
+rox_pixbuf_new_from_file_at_scale (const char *filename,
+				   int         width, 
+				   int         height,
+				   gboolean    preserve_aspect_ratio,
+				   GError    **error)
+{
+
+	GdkPixbufLoader *loader;
+	GdkPixbuf       *pixbuf;
+
+	guchar buffer [4096];
+	int length;
+	FILE *f;
+	struct {
+		gint width;
+		gint height;
+		gboolean preserve_aspect_ratio;
+	} info;
+
+	g_return_val_if_fail (filename != NULL, NULL);
+        g_return_val_if_fail (width > 0 && height > 0, NULL);
+
+	f = fopen (filename, "rb");
+	if (!f) {
+                gchar *utf8_filename = g_filename_to_utf8 (filename, -1,
+                                                           NULL, NULL, NULL);
+                g_set_error (error,
+                             G_FILE_ERROR,
+                             g_file_error_from_errno (errno),
+                             _("Failed to open file '%s': %s"),
+                             utf8_filename ? utf8_filename : "???",
+                             g_strerror (errno));
+                g_free (utf8_filename);
+		return NULL;
+        }
+
+	loader = gdk_pixbuf_loader_new ();
+
+	info.width = width;
+	info.height = height;
+        info.preserve_aspect_ratio = preserve_aspect_ratio;
+
+	g_signal_connect (loader, "size-prepared", G_CALLBACK (size_prepared_cb), &info);
+
+	while (!feof (f) && !ferror (f)) {
+		length = fread (buffer, 1, sizeof (buffer), f);
+		if (length > 0)
+			if (!gdk_pixbuf_loader_write (loader, buffer, length, error)) {
+				gdk_pixbuf_loader_close (loader, NULL);
+				fclose (f);
+				g_object_unref (loader);
+				return NULL;
+			}
+	}
+
+	fclose (f);
+
+	if (!gdk_pixbuf_loader_close (loader, error)) {
+		g_object_unref (loader);
+		return NULL;
+	}
+
+	pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
+
+	if (!pixbuf) {
+                gchar *utf8_filename = g_filename_to_utf8 (filename, -1,
+                                                           NULL, NULL, NULL);
+
+		g_object_unref (loader);
+
+                g_set_error (error,
+                             GDK_PIXBUF_ERROR,
+                             GDK_PIXBUF_ERROR_FAILED,
+                             _("Failed to load image '%s': reason not known, probably a corrupt image file"),
+                             utf8_filename ? utf8_filename : "???");
+                g_free (utf8_filename);
+		return NULL;
+	}
+
+	g_object_ref (pixbuf);
+
+	g_object_unref (loader);
+
+	return pixbuf;
+}
+
+/* Make the name bolder and larger.
+ * scale_factor can be PANGO_SCALE_X_LARGE, etc.
+ */
+void make_heading(GtkWidget *label, double scale_factor)
+{
+	PangoAttribute *attr;
+	PangoAttrList *list;
+
+	list = pango_attr_list_new();
+
+	attr = pango_attr_weight_new(PANGO_WEIGHT_BOLD);
+	attr->start_index = 0;
+	attr->end_index = -1;
+	pango_attr_list_insert(list, attr);
+
+	attr = pango_attr_scale_new(scale_factor);
+	attr->start_index = 0;
+	attr->end_index = -1;
+	pango_attr_list_insert(list, attr);
+
+	gtk_label_set_attributes(GTK_LABEL(label), list);
+}
+
+/* Launch a program using 0launch.
+ * If button-3 is used, open the GUI with -g.
+ */
+void launch_uri(const char *uri)
+{
+	const char *argv[] = {"0launch", NULL, NULL, NULL};
+	const char *uri_0launch = "/uri/0install/zero-install.sourceforge.net"
+				  "/bin/0launch";
+
+	if (!available_in_path(argv[0]))
+	{
+		if (access(uri_0launch, X_OK) == 0)
+			argv[0] = uri_0launch;
+		else
+		{
+			delayed_error(_("This program cannot be run, as the "
+				"0launch command is not available. "
+				"It can be downloaded from here:\n\n"
+				"http://0install.net/injector.html"));
+			return;
+		}
+	}
+
+	if (current_event_button() == 3)
+	{
+		argv[1] = "-g";
+		argv[2] = uri;
+	}
+	else
+		argv[1] = uri;
+
+	rox_spawn(NULL, argv);
+}
+
+static gint button3_button_pressed(GtkButton *button,
+				GdkEventButton *event,
+				gpointer date)
+{
+	if (event->button == 3)
+	{
+		gtk_grab_add(GTK_WIDGET(button));
+		gtk_button_pressed(button);
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+static gint button3_button_released(GtkButton *button,
+				GdkEventButton *event,
+				FilerWindow *filer_window)
+{
+	if (event->button == 3)
+	{
+		gtk_grab_remove(GTK_WIDGET(button));
+		gtk_button_released(button);
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+void allow_right_click(GtkWidget *button)
+{
+	g_signal_connect(button, "button_press_event",
+		G_CALLBACK(button3_button_pressed), NULL);
+	g_signal_connect(button, "button_release_event",
+		G_CALLBACK(button3_button_released), NULL);
+}
+
+/* Return mouse button used in the current event, or -1 if none (no event,
+ * or not a click).
+ */
+gint current_event_button(void)
+{
+	GdkEventButton *bev;
+	gint button = -1;
+
+	bev = (GdkEventButton *) gtk_get_current_event();
+
+	if (bev &&
+	    (bev->type == GDK_BUTTON_PRESS || bev->type == GDK_BUTTON_RELEASE))
+		button = bev->button;
+
+	gdk_event_free((GdkEvent *) bev);
+
+	return button;
+}
+
+/* Create a new pixbuf by colourizing 'src' to 'color'. If the function fails,
+ * 'src' will be returned (with an increased reference count, so it is safe to
+ * g_object_unref() the return value whether the function fails or not).
+ */
+GdkPixbuf *create_spotlight_pixbuf(GdkPixbuf *src, GdkColor *color)
+{
+	guchar opacity = 192;
+	guchar alpha = 255 - opacity;
+	GdkPixbuf *dst;
+	GdkColorspace colorspace;
+	int width, height, src_rowstride, dst_rowstride, x, y;
+	int n_channels, bps;
+	int r, g, b;
+	guchar *spixels, *dpixels, *src_pixels, *dst_pixels;
+	gboolean has_alpha;
+
+	has_alpha = gdk_pixbuf_get_has_alpha(src);
+	colorspace = gdk_pixbuf_get_colorspace(src);
+	n_channels = gdk_pixbuf_get_n_channels(src);
+	bps = gdk_pixbuf_get_bits_per_sample(src);
+
+	if ((colorspace != GDK_COLORSPACE_RGB) ||
+	    (!has_alpha && n_channels != 3) ||
+	    (has_alpha && n_channels != 4) ||
+	    (bps != 8))
+		goto error;
+
+	width = gdk_pixbuf_get_width(src);
+	height = gdk_pixbuf_get_height(src);
+
+	dst = gdk_pixbuf_new(colorspace, has_alpha, bps, width, height);
+	if (dst == NULL)
+		goto error;
+
+	src_pixels = gdk_pixbuf_get_pixels(src);
+	dst_pixels = gdk_pixbuf_get_pixels(dst);
+	src_rowstride = gdk_pixbuf_get_rowstride(src);
+	dst_rowstride = gdk_pixbuf_get_rowstride(dst);
+
+	r = opacity * (color->red >> 8);
+	g = opacity * (color->green >> 8);
+	b = opacity * (color->blue >> 8);
+
+	for (y = 0; y < height; y++)
+	{
+		spixels = src_pixels + y * src_rowstride;
+		dpixels = dst_pixels + y * dst_rowstride;
+		for (x = 0; x < width; x++)
+		{
+			*dpixels++ = (*spixels++ * alpha + r) >> 8;
+			*dpixels++ = (*spixels++ * alpha + g) >> 8;
+			*dpixels++ = (*spixels++ * alpha + b) >> 8;
+			if (has_alpha)
+				*dpixels++ = *spixels++;
+		}
+
+	}
+	return dst;
+
+error:
+	g_object_ref(src);
+	return src;
+}
