@@ -1,8 +1,8 @@
 /*
- * $Id: infobox.c,v 1.62 2004/07/30 19:53:10 tal197 Exp $
+ * $Id: infobox.c,v 1.75 2005/10/18 20:14:36 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2003, the ROX-Filer team.
+ * Copyright (C) 2005, the ROX-Filer team.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -46,6 +46,7 @@
 #include "xml.h"
 #include "mount.h"
 #include "pixmaps.h"
+#include "xtypes.h"
 
 typedef struct _FileStatus FileStatus;
 
@@ -133,7 +134,7 @@ void infobox_new(const gchar *pathname)
 			g_utf8_validate(path, -1, NULL) ? path
 							: _("(bad utf-8)"),
 				NULL, GTK_DIALOG_NO_SEPARATOR,
-				GTK_STOCK_CLOSE, GTK_RESPONSE_DELETE_EVENT,
+				GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL,
 				GTK_STOCK_REFRESH, GTK_RESPONSE_APPLY,
 				NULL);
 
@@ -225,7 +226,7 @@ static GtkWidget *make_vbox(const guchar *path, GObject *window)
 	hbox = gtk_hbox_new(FALSE, 4);
 	gtk_box_pack_start(vbox, hbox, FALSE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(hbox),
-			   gtk_image_new_from_pixbuf(item->image->pixbuf),
+			   gtk_image_new_from_pixbuf(di_image(item)->pixbuf),
 			   FALSE, FALSE, 4);
 
 	if (g_utf8_validate(item->leafname, -1, NULL))
@@ -239,27 +240,10 @@ static GtkWidget *make_vbox(const guchar *path, GObject *window)
 		g_free(u8);
 	}
 	gtk_label_set_selectable(GTK_LABEL(name), TRUE);
+	gtk_label_set_line_wrap(GTK_LABEL(name), TRUE);
 	gtk_box_pack_start(GTK_BOX(hbox), name, FALSE, TRUE, 4);
 	
-	/* Make the name bolder and larger */
-	{
-		PangoAttribute *attr;
-		PangoAttrList *list;
-
-		list = pango_attr_list_new();
-
-		attr = pango_attr_weight_new(PANGO_WEIGHT_BOLD);
-		attr->start_index = 0;
-		attr->end_index = -1;
-		pango_attr_list_insert(list, attr);
-
-		attr = pango_attr_scale_new(PANGO_SCALE_X_LARGE);
-		attr->start_index = 0;
-		attr->end_index = -1;
-		pango_attr_list_insert(list, attr);
-		
-		gtk_label_set_attributes(GTK_LABEL(name), list);
-	}
+	make_heading(name, PANGO_SCALE_X_LARGE);
 
 	/* List of file attributes */
 	add_frame(vbox, make_details(path, item, window));
@@ -282,13 +266,17 @@ static GtkWidget *make_vbox(const guchar *path, GObject *window)
 	}
 	g_free(help_dir);
 
-	label = gtk_label_new(NULL);
-	gtk_label_set_markup(GTK_LABEL(label), _("<b>Permissions</b>"));
-	gtk_misc_set_alignment(GTK_MISC(label), 0, 1);
-	gtk_box_pack_start(vbox, label, FALSE, TRUE, 2);
+	if (!(item->flags & ITEM_FLAG_SYMLINK))
+	{
+		label = gtk_label_new(NULL);
+		gtk_label_set_markup(GTK_LABEL(label),
+				     _("<b>Permissions</b>"));
+		gtk_misc_set_alignment(GTK_MISC(label), 0, 1);
+		gtk_box_pack_start(vbox, label, FALSE, TRUE, 2);
 
-	gtk_box_pack_start(vbox, make_permissions(path, item),
-			FALSE, TRUE, 0);
+		gtk_box_pack_start(vbox, make_permissions(path, item),
+				   FALSE, TRUE, 0);
+	}
 
 	if (about)
 		add_frame(vbox, make_about(path, ai));
@@ -514,20 +502,24 @@ static void cell_edited(GtkCellRendererText *cell,
 
 	oldlink = readlink_dup(fullpath);
 	if (!oldlink) {
-		report_error(_("'%s' is no longer a symlink"), fullpath);
+		/* Must use delayed_error(), as this can be called
+		 * from a focus-out event (causes a crash).
+		 */
+		delayed_error(_("'%s' is no longer a symlink"), fullpath);
 		return;
 	}
 	if (strcmp(oldlink, new_text) == 0)
 		return;	/* No change */
 	g_free(oldlink);
 	if (unlink(fullpath)) {
-		report_error(_("Failed to unlink '%s':\n%s"),
+		delayed_error(_("Failed to unlink '%s':\n%s"),
 				fullpath, g_strerror(errno));
 		return;
 	}
 	if (symlink(new_text, fullpath)) {
-		report_error(_("Failed to create symlink from '%s':\n%s\n"
-				"(note: old link has been deleted)"));
+		delayed_error(_("Failed to create symlink from '%s':\n%s\n"
+				"(note: old link has been deleted)"),
+				fullpath, g_strerror(errno));
 		return;
 	}
 
@@ -630,6 +622,14 @@ static GtkWidget *make_details(const guchar *path, DirItem *item,
 	if (item->mime_type)
 		add_row(store, "", mime_type_comment(item->mime_type));
 
+	if (xattr_supported(NULL)) {
+		add_row(store, _("Extended attributes:"),
+		  (item->flags & ITEM_FLAG_HAS_XATTR)
+		  	? _("Present")
+			: xattr_supported(path) ? _("None")
+						: _("Not supported"));
+	}
+
 	if (item->flags & ITEM_FLAG_SYMLINK)
 	{
 		GtkTreeIter iter;
@@ -649,8 +649,15 @@ static GtkWidget *make_details(const guchar *path, DirItem *item,
 	}
 
 	if (item->base_type != TYPE_DIRECTORY)
-		add_row_and_free(store, _("Run action:"),
-				 describe_current_command(item->mime_type));
+	{
+		if (EXECUTABLE_FILE(item))
+			add_row(store, _("Run action:"), _("Execute file"));
+		else
+		{
+			add_row_and_free(store, _("Run action:"),
+				describe_current_command(item->mime_type));
+		}
+	}
 
 	return view;
 }
