@@ -1,8 +1,8 @@
 /*
- * $Id: menu.c,v 1.282 2004/05/29 14:04:36 kerofin Exp $
+ * $Id: menu.c,v 1.295 2005/11/29 20:24:49 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2003, the ROX-Filer team.
+ * Copyright (C) 2005, the ROX-Filer team.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -61,6 +61,8 @@
 #include "display.h"
 #include "bookmarks.h"
 #include "panel.h"
+#include "bulk_rename.h"
+#include "xtypes.h"
 
 typedef enum {
 	FILE_COPY_ITEM,
@@ -75,7 +77,6 @@ typedef enum {
 	FILE_USAGE,
 	FILE_CHMOD_ITEMS,
 	FILE_FIND,
-	FILE_OPEN_VFS_AVFS,
 	FILE_SET_TYPE,
 } FileOp;
 
@@ -140,6 +141,7 @@ static void clear_selection(gpointer data, guint action, GtkWidget *widget);
 static void invert_selection(gpointer data, guint action, GtkWidget *widget);
 static void new_directory(gpointer data, guint action, GtkWidget *widget);
 static void new_file(gpointer data, guint action, GtkWidget *widget);
+static void customise_new(gpointer data);
 static void xterm_here(gpointer data, guint action, GtkWidget *widget);
 
 static void open_parent_same(gpointer data, guint action, GtkWidget *widget);
@@ -168,6 +170,7 @@ static GtkWidget	*filer_thumb_menu;	/* The Show Thumbs item */
 static GtkWidget	*filer_new_window;	/* The New Window item */
 static GtkWidget        *filer_new_menu;        /* The New submenu */
 static GtkWidget        *filer_follow_sym;      /* Follow symbolic links item */
+static GtkWidget        *filer_set_type;        /* Set type item */
 
 #undef N_
 #define N_(x) x
@@ -206,7 +209,6 @@ static GtkItemFactoryEntry filer_menu_def[] = {
 {">" N_("Delete"),	    	"<Ctrl>X", file_op, FILE_DELETE, "<StockItem>", GTK_STOCK_DELETE},
 {">",				NULL, NULL, 0, "<Separator>"},
 {">" N_("Shift Open"),   	NULL, file_op, FILE_OPEN_FILE},
-{">" N_("Open AVFS"),		NULL, file_op, FILE_OPEN_VFS_AVFS, "<StockItem>", GTK_STOCK_OPEN},
 {">" N_("Send To..."),		NULL, file_op, FILE_SEND_TO, NULL},
 {">",				NULL, NULL, 0, "<Separator>"},
 {">" N_("Set Run Action..."),	"asterisk", file_op, FILE_RUN_ACTION, "<StockItem>", GTK_STOCK_EXECUTE},
@@ -227,6 +229,7 @@ static GtkItemFactoryEntry filer_menu_def[] = {
 {N_("New"),			NULL, NULL, 0, "<Branch>"},
 {">" N_("Directory"),		NULL, new_directory, 0, NULL},
 {">" N_("Blank file"),		NULL, new_file, 0, "<StockItem>", GTK_STOCK_NEW},
+{">" N_("Customise Menu..."),	NULL, customise_new, 0, NULL},
 {N_("Window"),			NULL, NULL, 0, "<Branch>"},
 {">" N_("Parent, New Window"), 	NULL, open_parent, 0, "<StockItem>", GTK_STOCK_GO_UP},
 {">" N_("Parent, Same Window"), NULL, open_parent_same, 0, NULL},
@@ -241,8 +244,8 @@ static GtkItemFactoryEntry filer_menu_def[] = {
 {">",				NULL, NULL, 0, "<Separator>"},
 {">" N_("Enter Path..."),	"slash", mini_buffer, MINI_PATH, NULL},
 {">" N_("Shell Command..."),	"<Shift>exclam", mini_buffer, MINI_SHELL, NULL},
-{">" N_("Xterm Here"),		"grave", xterm_here, FALSE, NULL},
-{">" N_("Switch to xterm"),	NULL, xterm_here, TRUE, NULL},
+{">" N_("Terminal Here"),	"grave", xterm_here, FALSE, NULL},
+{">" N_("Switch to Terminal"),	NULL, xterm_here, TRUE, NULL},
 {N_("Help"),			NULL, NULL, 0, "<Branch>"},
 {">" N_("About ROX-Filer..."),	NULL, menu_rox_help, HELP_ABOUT, NULL},
 {">" N_("Show Help Files"),	"F1", menu_rox_help, HELP_DIR, "<StockItem>", GTK_STOCK_HELP},
@@ -290,6 +293,8 @@ gboolean ensure_filer_menu(void)
 	GET_SSMENU_ITEM(filer_auto_size_menu, "filer", "Display", "Automatic");
 	GET_SSMENU_ITEM(filer_thumb_menu, "filer", "Display",
 							"Show Thumbnails");
+	GET_SSMENU_ITEM(item, "filer", "File", "Set Type...");
+	filer_set_type = GTK_BIN(item)->child;
 
 	GET_SMENU_ITEM(filer_new_menu, "filer", "New");
 	GET_SSMENU_ITEM(item, "filer", "Window", "Follow Symbolic Links");
@@ -323,7 +328,7 @@ void menu_init(void)
 {
 	char			*menurc;
 
-	menurc = choices_find_path_load(MENUS_NAME, PROJECT);
+	menurc = choices_find_xdg_path_load(MENUS_NAME, PROJECT, SITE);
 	if (menurc)
 	{
 		gtk_accel_map_load(menurc);
@@ -378,9 +383,10 @@ static void menuitem_no_shortcuts(GtkWidget *item)
 /* Shade items that only work on single files */
 static void shade_file_menu_items(gboolean shaded)
 {
-	menu_set_items_shaded(filer_file_menu, shaded, 0, 3);
-	menu_set_items_shaded(filer_file_menu, shaded, 5, 2);
-	menu_set_items_shaded(filer_file_menu, shaded, 9, 2);
+	menu_set_items_shaded(filer_file_menu, shaded, 0, 1);
+	menu_set_items_shaded(filer_file_menu, shaded, 2, 1);
+	menu_set_items_shaded(filer_file_menu, shaded, 5, 1);
+	menu_set_items_shaded(filer_file_menu, shaded, 8, 2);
 }
 
 /* 'data' is an array of three ints:
@@ -428,19 +434,22 @@ GtkWidget *make_send_to_item(DirItem *ditem, const char *label,
 {
 	GtkWidget *item;
 	
-	if (ditem->image && style != MIS_NONE)
+	if (style != MIS_NONE && di_image(ditem))
 	{
 		GdkPixbuf *pixbuf;
+		MaskedPixmap *image;
+
+		image = di_image(ditem);
 
 		switch (style)
 		{
 			case MIS_LARGE:
-				pixbuf = ditem->image->pixbuf;
+				pixbuf = image->pixbuf;
 				break;
 			default:
-				if (!ditem->image->sm_pixbuf)
-					pixmap_make_small(ditem->image);
-				pixbuf = ditem->image->sm_pixbuf;
+				if (!image->sm_pixbuf)
+					pixmap_make_small(image);
+				pixbuf = image->sm_pixbuf;
 				break;
 		}
 
@@ -476,18 +485,19 @@ static GList *menu_from_dir(GtkWidget *menu, const gchar *dir_name,
 	if (!names)
 		goto out;
 
-	if (separator)
-	{
-		item = gtk_menu_item_new();
-		widgets = g_list_append(widgets, item);
-		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-	}
-
 	for (i = 0; i < names->len; i++)
 	{
 		char	*leaf = names->pdata[i];
 		gchar	*fname;
 
+		if (separator)
+		{
+			item = gtk_menu_item_new();
+			widgets = g_list_append(widgets, item);
+			gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+			separator = FALSE;
+		}
+
 		fname = g_strconcat(dname, "/", leaf, NULL);
 
 		/* Strip off extension, if any */
@@ -517,7 +527,7 @@ static GList *menu_from_dir(GtkWidget *menu, const gchar *dir_name,
 
 			sub = gtk_menu_new();
 			new_widgets = menu_from_dir(sub, fname, style, func,
-						separator, strip_ext, FALSE);
+						separator, strip_ext, TRUE);
 			g_list_free(new_widgets);
 			gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), sub);
 		}
@@ -559,7 +569,7 @@ static void update_new_files_menu(MenuIconStyle style)
 		widgets = NULL;
 	}
 
-	templ_dname = choices_find_path_load("Templates", "");
+	templ_dname = choices_find_xdg_path_load("Templates", "", SITE);
 	if (templ_dname)
 	{
 		widgets = menu_from_dir(filer_new_menu, templ_dname, style,
@@ -664,6 +674,8 @@ void show_filer_menu(FilerWindow *filer_window, GdkEvent *event, ViewIter *iter)
 	int		n_selected;
 	int             n_added = 0;
 
+	g_return_if_fail(event != NULL);
+
 	n_selected = view_count_selected(filer_window->view);
 
 	ensure_filer_menu();
@@ -675,9 +687,6 @@ void show_filer_menu(FilerWindow *filer_window, GdkEvent *event, ViewIter *iter)
 
 	window_with_focus = filer_window;
 
-	if (!event)
-		event = gtk_get_current_event();
-
 	if (event->type == GDK_BUTTON_PRESS)
 		state = ((GdkEventButton *) event)->state;
 	else if (event->type == GDK_KEY_PRESS)
@@ -745,7 +754,7 @@ void show_filer_menu(FilerWindow *filer_window, GdkEvent *event, ViewIter *iter)
 				break;
 			case 1:
 				item = filer_selected_item(filer_window);
-				if (!item->image)
+				if (item->base_type == TYPE_UNKNOWN)
 					dir_update_item(filer_window->directory,
 							item->leafname);
 				shade_file_menu_items(FALSE);
@@ -758,7 +767,7 @@ void show_filer_menu(FilerWindow *filer_window, GdkEvent *event, ViewIter *iter)
 						: _("(bad utf-8)"));
 				if (!can_set_run_action(file_item))
 					menu_set_items_shaded(filer_file_menu,
-							TRUE, 9, 1);
+							TRUE, 8, 1);
 				break;
 			default:
 				shade_file_menu_items(TRUE);
@@ -783,6 +792,8 @@ void show_filer_menu(FilerWindow *filer_window, GdkEvent *event, ViewIter *iter)
 			!o_unique_filer_windows.int_value);
 	gtk_widget_set_sensitive(filer_follow_sym,
 		strcmp(filer_window->sym_path, filer_window->real_path) != 0);
+	gtk_widget_set_sensitive(filer_set_type,
+				 xattr_supported(filer_window->real_path));
 
 	if (n_selected && o_menu_quick.int_value) 
 		popup_menu = (state & GDK_CONTROL_MASK)
@@ -1004,9 +1015,26 @@ static void chmod_items(FilerWindow *filer_window)
 
 static void set_type_items(FilerWindow *filer_window)
 {
-	GList *paths;
+	GList *paths, *p;
+	int npass=0, nfail=0;
+	
 	paths = filer_selected_items(filer_window);
-	action_settype(paths, FALSE, NULL);
+	for(p=paths; p; p=g_list_next(p)) {
+		if(xattr_supported((const char *) p->data))
+			npass++;
+		else
+			nfail++;
+	}
+	if(npass==0) 
+		report_error(_("Extended attributes, used to store types, are not supported for this "
+				"file or files.\n"
+				"This may be due to lack of support from the filesystem or the C library, "
+				"or it may simply be that the filesystem needs to be mounted with "
+				"the right mount option ('user_xattr' on Linux)."));
+	else if(nfail>0)
+		report_error(_("Setting type not supported for some of these files"));
+	if(npass>0)
+		action_settype(paths, FALSE, NULL);
 	destroy_glist(&paths);
 }
 
@@ -1236,17 +1264,6 @@ void open_home(gpointer data, guint action, GtkWidget *widget)
 	filer_opendir(home_dir, NULL, NULL);
 }
 
-static void open_vfs_avfs(FilerWindow *filer_window, DirItem *item)
-{
-	gchar		*path;
-
-	path = g_strconcat(filer_window->sym_path,
-			"/", item->leafname, "#", NULL);
-
-	filer_change_to(filer_window, path, NULL);
-	g_free(path);
-}
-
 static void select_all(gpointer data, guint action, GtkWidget *widget)
 {
 	g_return_if_fail(window_with_focus != NULL);
@@ -1374,7 +1391,7 @@ static gboolean new_file_type_cb(GObject *savebox,
 
 	/* We can work out the template path from the initial name */
 	oleaf = g_basename(initial);
-	templ_dname = choices_find_path_load("Templates", "");
+	templ_dname = choices_find_xdg_path_load("Templates", "", SITE);
 	if (!templ_dname)
 	{
 		report_error(
@@ -1434,17 +1451,17 @@ static void customise_send_to(gpointer data)
 
 	dirs = g_string_new(NULL);
 
-	path = choices_list_dirs("");
+	path = choices_list_xdg_dirs("", SITE);
 	for (i = 0; i < path->len; i++)
 	{
 		guchar *old = (guchar *) path->pdata[i];
 
 		g_string_append(dirs, old);
-		g_string_append(dirs, "SendTo\n");
+		g_string_append(dirs, "/SendTo\n");
 	}
 	choices_free_list(path);
 
-	save = choices_find_path_save("", "SendTo", TRUE);
+	save = choices_find_xdg_path_save("", "SendTo", SITE, TRUE);
 	if (save)
 		mkdir(save, 0777);
 
@@ -1472,6 +1489,46 @@ static void customise_send_to(gpointer data)
 		filer_opendir(save, NULL, NULL);
 }
 
+static void customise_new(gpointer data)
+{
+	GPtrArray	*path;
+	guchar		*save;
+	GString		*dirs;
+	int		i;
+
+	dirs = g_string_new(NULL);
+
+	path = choices_list_xdg_dirs("", SITE);
+	for (i = 0; i < path->len; i++)
+	{
+		guchar *old = (guchar *) path->pdata[i];
+
+		g_string_append(dirs, old);
+		g_string_append(dirs, "/Templates\n");
+	}
+	choices_free_list(path);
+
+	save = choices_find_xdg_path_save("", "Templates", SITE, TRUE);
+	if (save)
+		mkdir(save, 0777);
+
+	info_message(
+		_("Any files placed in your Templates directories will "
+		"appear on the `New' menu. Choosing one of them will make "
+		"a copy of it as the new file.\n\n"
+		"The following directories contain templates:\n\n%s\n%s\n"),
+		dirs->str,
+		save ? _("I'll show you your Templates directory now; you "
+			 "should place any template files you want inside it.")
+		     : _("Your CHOICESPATH variable setting prevents "
+			 "customisations - sorry."));
+
+	g_string_free(dirs, TRUE);
+	
+	if (save)
+		filer_opendir(save, NULL, NULL);
+}
+
 /* Add everything in the directory <Choices>/SendTo/[.type[_subtype]] 
  * to the menu.
  */
@@ -1488,7 +1545,7 @@ static void add_sendto(GtkWidget *menu, const gchar *type, const gchar *subtype)
 	else
 		searchdir = g_strdup("SendTo");
 
-	paths = choices_list_dirs(searchdir);
+	paths = choices_list_xdg_dirs(searchdir, SITE);
 	g_free(searchdir);	
 
 	for (i = 0; i < paths->len; i++)
@@ -1605,7 +1662,8 @@ static void send_to(FilerWindow *filer_window)
 	/* Eats paths */
 	show_send_to_menu(paths, event);
 
-	gdk_event_free(event);
+	if (event)
+		gdk_event_free(event);
 }
 
 static void xterm_here(gpointer data, guint action, GtkWidget *widget)
@@ -1718,7 +1776,12 @@ void menu_rox_help(gpointer data, guint action, GtkWidget *widget)
 		{
 			manual = g_strconcat(app_dir, "/Help/Manual-",
 					     current_lang, ".html", NULL);
-			if (access(manual, F_OK))
+			if (!file_exists(manual) && strchr(current_lang, '_'))
+			{
+				/* Try again without the territory */
+				strcpy(strrchr(manual, '_'), ".html");
+			}
+			if (!file_exists(manual))
 				null_g_free(&manual);
 		}
 
@@ -1754,7 +1817,7 @@ static void save_menus(void)
 {
 	char	*menurc;
 
-	menurc = choices_find_path_save(MENUS_NAME, PROJECT, TRUE);
+	menurc = choices_find_xdg_path_save(MENUS_NAME, PROJECT, SITE, TRUE);
 	if (menurc)
 	{
 		gtk_accel_map_save(menurc);
@@ -1833,9 +1896,6 @@ static void file_op(gpointer data, FileOp action, GtkWidget *unused)
 			case FILE_FIND:
 				prompt = _("Search inside ... ?");
 				break;
-			case FILE_OPEN_VFS_AVFS:
-				prompt = _("Look inside ... ?");
-				break;
 			default:
 				g_warning("Unknown action!");
 				return;
@@ -1874,6 +1934,22 @@ static void file_op(gpointer data, FileOp action, GtkWidget *unused)
 			destroy_glist(&items);
 			return;
 		}
+		case FILE_RENAME_ITEM:
+			if (n_selected > 1)
+			{
+				GList *items = NULL;
+				ViewIter iter;
+
+				view_get_iter(window_with_focus->view, &iter, VIEW_ITER_SELECTED);
+				while ((item = iter.next(&iter)))
+					items = g_list_prepend(items, item->leafname);
+				items = g_list_reverse(items);
+
+				bulk_rename(window_with_focus->sym_path, items);
+				g_list_free(items);
+				return;
+			}
+			break;	/* Not a bulk rename... see below */
 		default:
 			break;
 	}
@@ -1893,7 +1969,7 @@ static void file_op(gpointer data, FileOp action, GtkWidget *unused)
 	g_return_if_fail(item != NULL);
 	/* iter may be passed to filer_openitem... */
 
-	if (!item->image)
+	if (item->base_type == TYPE_UNKNOWN)
 		item = dir_update_item(window_with_focus->directory,
 					item->leafname);
 
@@ -1908,17 +1984,17 @@ static void file_op(gpointer data, FileOp action, GtkWidget *unused)
 	switch (action)
 	{
 		case FILE_COPY_ITEM:
-			src_dest_action_item(path, item->image,
+			src_dest_action_item(path, di_image(item),
 					_("Copy"), copy_cb,
 					GDK_ACTION_COPY);
 			break;
 		case FILE_RENAME_ITEM:
-			src_dest_action_item(path, item->image,
+			src_dest_action_item(path, di_image(item),
 					_("Rename"), rename_cb,
 					GDK_ACTION_MOVE);
 			break;
 		case FILE_LINK_ITEM:
-			src_dest_action_item(path, item->image,
+			src_dest_action_item(path, di_image(item),
 					_("Symlink"), link_cb,
 					GDK_ACTION_LINK);
 			break;
@@ -1932,9 +2008,6 @@ static void file_op(gpointer data, FileOp action, GtkWidget *unused)
 		case FILE_SET_ICON:
 			icon_set_handler_dialog(item, path);
 			break;
-		case FILE_OPEN_VFS_AVFS:
-			open_vfs_avfs(window_with_focus, item);
-			break;
 		default:
 			g_warning("Unknown action!");
 			return;
