@@ -1,8 +1,8 @@
 /*
- * $Id: minibuffer.c,v 1.70 2004/04/21 09:07:16 tal197 Exp $
+ * $Id: minibuffer.c,v 1.76 2005/08/15 18:51:16 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2003, the ROX-Filer team.
+ * Copyright (C) 2005, the ROX-Filer team.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -30,6 +30,9 @@
 #include <glob.h>
 #include <stdio.h>
 
+#include <sys/types.h>
+#include <pwd.h>
+ 
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
@@ -360,7 +363,7 @@ static void complete(FilerWindow *filer_window)
 	
 	current_stem = strlen(leaf);
 
-	/* Find the longest other match of this name. It it's longer than
+	/* Find the longest other match of this name. If it's longer than
 	 * the currently entered text then complete only up to that length.
 	 */
 	view_get_iter(filer_window->view, &iter, 0);
@@ -373,7 +376,13 @@ static void complete(FilerWindow *filer_window)
 
 		while (other->leafname[stem] && item->leafname[stem])
 		{
-			if (other->leafname[stem] != item->leafname[stem])
+			gchar	a, b;
+			/* Like the matches() function below, the comparison of
+			 * leafs must be case-insensitive.
+			 */
+		  	a = g_ascii_tolower(item->leafname[stem]);
+		  	b = g_ascii_tolower(other->leafname[stem]);
+			if (a != b)
 				break;
 			stem++;
 		}
@@ -392,17 +401,23 @@ static void complete(FilerWindow *filer_window)
 	}
 	else if (current_stem < shortest_stem)
 	{
-		guchar	*extra;
 		gint tmp_pos;
-
-		extra = g_strndup(item->leafname + current_stem,
-				shortest_stem - current_stem);
-
-		tmp_pos = entry->text_length;
-		gtk_editable_insert_text(GTK_EDITABLE(entry), extra, -1,
+		
+		/* Have to replace the leafname text in the minibuffer rather
+		 * than just append to it.  Here's an example:
+		 * Suppose we have two dirs, /tmp and /TMP.
+		 * The user enters /t in the minibuffer and hits Tab.
+		 * With the g_ascii_tolower() code above, this would result
+		 * in /tMP being bisplayed in the minibuffer which isn't
+		 * intuitive.  Therefore all text after the / must be replaced.
+		 */
+		tmp_pos = leaf - text; /* index of start of leaf */
+		gtk_editable_delete_text(GTK_EDITABLE(entry),
+					 tmp_pos, entry->text_length);
+		gtk_editable_insert_text(GTK_EDITABLE(entry),
+					 item->leafname, shortest_stem,
 					 &tmp_pos);
 		
-		g_free(extra);
 		gtk_editable_set_position(GTK_EDITABLE(entry), -1);
 
 		if (o_filer_beep_multi.int_value)
@@ -426,13 +441,13 @@ static void complete(FilerWindow *filer_window)
 static void path_changed(FilerWindow *filer_window)
 {
 	GtkWidget *mini = filer_window->minibuffer;
-	const char	*new, *leaf;
+	const char	*rawnew, *leaf;
 	char		*path;
+	char		*new = NULL;
 	gboolean	error = FALSE;
 
-	new = gtk_entry_get_text(GTK_ENTRY(mini));
-
-	if (!*new)
+	rawnew = gtk_entry_get_text(GTK_ENTRY(mini));
+	if (!*rawnew)
 	{
 		/* Entry may be blank because we're in the middle of changing
 		 * to something else...
@@ -441,6 +456,50 @@ static void path_changed(FilerWindow *filer_window)
 		return;
 	}
 
+	switch (rawnew[0])
+	{
+	case '/':
+		new=g_strdup(rawnew);
+		break;
+
+	case '~':
+		if (!rawnew[1] || rawnew[1]=='/')
+		{
+			new=g_strconcat(g_get_home_dir(), "/", 
+					     rawnew[1]? rawnew+2: "", "/", 
+					     NULL);
+		}
+		else
+		{
+			const char *sl;
+			gchar *username;
+			struct passwd *passwd;
+
+			
+			/* Need to lookup user name */
+			for(sl=rawnew+2; *sl && *sl!='/'; sl++)
+				;
+			username=g_strndup(rawnew+1, sl-rawnew-1);
+			passwd=getpwnam(username);
+			g_free(username);
+
+			if(passwd)
+			{
+				new=g_strconcat(passwd->pw_dir, "/", 
+					     sl+1, "/", 
+					     NULL);
+			}
+			else
+				new=g_strdup(rawnew);
+		}
+		break;
+
+	default:
+		new=g_strdup(rawnew);
+		break;
+	}
+		
+
 	leaf = g_basename(new);
 	if (leaf == new)
 		path = g_strdup("/");
@@ -462,7 +521,7 @@ static void path_changed(FilerWindow *filer_window)
 	}
 	else
 	{
-		if (*leaf == '.')
+		if (*leaf == '.' && !filer_window->temp_show_hidden)
 		{
 			filer_window->temp_show_hidden = TRUE;
 			display_update_hidden(filer_window);
@@ -473,6 +532,7 @@ static void path_changed(FilerWindow *filer_window)
 			error = TRUE;
 	}
 		
+	g_free(new);
 	g_free(path);
 
 	entry_set_error(mini, error);
@@ -552,7 +612,7 @@ static gboolean matches(ViewIter *iter, const char *pattern)
 	
 	item = iter->peek(iter);
 	
-	return strncmp(item->leafname, pattern, strlen(pattern)) == 0;
+	return strncasecmp(item->leafname, pattern, strlen(pattern)) == 0;
 }
 
 /* Find next match and set base for future matches. */
