@@ -1,6 +1,4 @@
 /*
- * $Id: panel.h,v 1.11 2002/01/03 14:29:15 tal197 Exp $
- *
  * ROX-Filer, filer for the ROX desktop project
  * By Thomas Leonard, <tal197@users.sourceforge.net>.
  */
@@ -8,17 +6,19 @@
 #ifndef _PANEL_H
 #define _PANEL_H
 
-#define MENU_MARGIN 32
-
 typedef enum {
 	PANEL_TOP,
 	PANEL_BOTTOM,
 	PANEL_LEFT,
 	PANEL_RIGHT,
 
-	PANEL_NUMBER_OF_SIDES	/* This goes last! */
+	PANEL_NUMBER_OF_SIDES,	/* (goes after valid sides) */
+
+	PANEL_DEFAULT_SIDE	/* Read from file, or use free side */
 } PanelSide;
 
+#define MENU_MARGIN(side) ((side) == PANEL_BOTTOM ? 32 : 8)
+
 struct _Panel {
 	GtkWidget	*window;
 	GtkAdjustment	*adj;		/* Scroll position of the bar */
@@ -29,15 +29,24 @@ struct _Panel {
 	GtkWidget	*after;		/* Icons at the right/bottom end */
 
 	GtkWidget	*gap;		/* Event box between sides */
+
+	int		autoscroll_to;	/* Timeout */
+	int		autoscroll_speed; /* 0 => not scrolling */
 };
 
 extern Panel *current_panel[PANEL_NUMBER_OF_SIDES];
 
 void panel_init(void);
-Panel *panel_new(guchar *name, PanelSide side);
+Panel *panel_new(const gchar *name, PanelSide side);
 void panel_icon_may_update(Icon *icon);
 void panel_save(Panel *panel);
-gboolean panel_want_show_text(Icon *icon);
-void panel_icon_renamed(Icon *icon);
+
+gboolean panel_add(PanelSide side,
+		   const gchar *path, const gchar *label, gboolean after, const gchar *shortcut, const gchar *args);
+gboolean panel_remove_item(PanelSide side, const gchar *path,
+			   const gchar *label);
+void panel_mark_used(GdkRegion *used);
+void panel_update_size(void);
+PanelSide panel_name_to_side(gchar *side);
 
 #endif /* _PANEL_H */
