@@ -1,5 +1,5 @@
 /*
- * $Id: pinboard.c,v 1.186 2005/01/16 16:48:01 tal197 Exp $
+ * $Id: pinboard.c,v 1.191 2005/12/16 20:57:53 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2005, the ROX-Filer team.
@@ -141,6 +141,7 @@ static int lasso_rect_y1, lasso_rect_y2;
 /* Tracking icon positions */
 static GHashTable *placed_icons=NULL;
 
+Option o_pinboard_tasklist_per_workspace;
 static Option o_pinboard_clamp_icons, o_pinboard_grid_step;
 static Option o_pinboard_fg_colour, o_pinboard_bg_colour;
 static Option o_pinboard_tasklist, o_forward_buttons_13;
@@ -229,8 +230,9 @@ static void update_pinboard_font(void);
 static void draw_lasso(void);
 static gint lasso_motion(GtkWidget *widget, GdkEventMotion *event, gpointer d);
 static void clear_backdrop(GtkWidget *drop_box, gpointer data);
-static void radios_changed(gpointer data);
+static void radios_changed(Radios *radios, gpointer data);
 static void update_radios(GtkWidget *dialog);
+static void pinboard_set_backdrop_box(void);
 
 /****************************************************************
  *			EXTERNAL INTERFACE			*
@@ -249,6 +251,7 @@ void pinboard_init(void)
 	option_add_int(&o_pinboard_grid_step, "pinboard_grid_step",
 							GRID_STEP_COARSE);
 	option_add_int(&o_pinboard_tasklist, "pinboard_tasklist", TRUE);
+	option_add_int(&o_pinboard_tasklist_per_workspace, "pinboard_tasklist_per_workspace", FALSE);
 	option_add_int(&o_forward_buttons_13, "pinboard_forward_buttons_13",
 			FALSE);
 
@@ -315,7 +318,7 @@ void pinboard_activate(const gchar *name)
 		guchar	*leaf;
 
 		leaf = g_strconcat("pb_", name, NULL);
-		path = choices_find_path_load(leaf, PROJECT);
+		path = choices_find_xdg_path_load(leaf, PROJECT, SITE);
 		g_free(leaf);
 	}
 
@@ -448,7 +451,7 @@ void pinboard_moved_widget(GtkWidget *widget, const gchar *name,
  * 'x' and 'y' are the coordinates of the point in the middle of the text.
  * 'name' is the name to use. If NULL then the leafname of path is used.
  */
-static void pinboard_pin_with_args(const gchar *path, const gchar *name,
+void pinboard_pin_with_args(const gchar *path, const gchar *name,
 				   int x, int y, const gchar *shortcut,
 				   const gchar *args)
 {
@@ -657,7 +660,7 @@ void pinboard_set_backdrop_app(const gchar *app)
 }
 
 /* Open a dialog box allowing the user to set the backdrop */
-void pinboard_set_backdrop_box(void)
+static void pinboard_set_backdrop_box(void)
 {
 	GtkWidget *dialog, *frame, *label, *hbox;
 	GtkBox *vbox;
@@ -965,6 +968,7 @@ static gint draw_icon(GtkWidget *widget, GdkEventExpose *event, PinIcon *pi)
 	int		iwidth = image->width;
 	int		iheight = image->height;
 	int		x, y;
+	GdkPixbuf	*pixbuf;
 	
 	if (!parent_class)
 	{
@@ -977,11 +981,19 @@ static gint draw_icon(GtkWidget *widget, GdkEventExpose *event, PinIcon *pi)
 
 	gdk_gc_set_clip_region(pi->widget->style->black_gc, event->region);
 
-	render_pixbuf(icon->selected ? image->pixbuf_lit : image->pixbuf,
+	pixbuf = icon->selected
+		? create_spotlight_pixbuf(image->pixbuf,
+			&pi->widget->style->base[GTK_STATE_SELECTED])
+		: image->pixbuf;
+
+	render_pixbuf(pixbuf,
 			pi->widget->window,
 			pi->widget->style->black_gc,
 			x, y, iwidth, iheight);
 
+	if (icon->selected)
+		g_object_unref(pixbuf);
+
 	if (item->flags & ITEM_FLAG_SYMLINK)
 	{
 		render_pixbuf(im_symlink->pixbuf, pi->widget->window,
@@ -1506,7 +1518,7 @@ static void pinboard_save(void)
 		guchar	*leaf;
 
 		leaf = g_strconcat("pb_", current_pinboard->name, NULL);
-		save = choices_find_path_save(leaf, PROJECT, TRUE);
+		save = choices_find_xdg_path_save(leaf, PROJECT, SITE, TRUE);
 		g_free(leaf);
 	}
 
@@ -2096,13 +2108,17 @@ static void pin_icon_set_tip(PinIcon *pi)
 
 static void pinboard_show_menu(GdkEventButton *event, PinIcon *pi)
 {
+	GtkWidget	*option_item;
 	int		pos[3];
 	GList		*list;
 
 	pos[0] = event->x_root;
 	pos[1] = event->y_root;
 
-	icon_prepare_menu((Icon *) pi, TRUE);
+	option_item = gtk_image_menu_item_new_with_label(_("Backdrop..."));
+	g_signal_connect(option_item, "activate",
+			 G_CALLBACK(pinboard_set_backdrop_box), NULL);
+	icon_prepare_menu((Icon *) pi, option_item);
 
 	list = gtk_container_get_children(GTK_CONTAINER(icon_menu));
 	pos[2] = g_list_length(list) - 6;
@@ -2614,16 +2630,14 @@ static void update_pinboard_font(void)
 			: NULL;
 }
 
-static void radios_changed(gpointer data)
+static void radios_changed(Radios *radios, gpointer data)
 {
 	GObject *dialog = G_OBJECT(data);
 	DropBox *drop_box;
 	const guchar *path;
-	Radios *radios;
 
 	g_return_if_fail(dialog != NULL);
 
-	radios = g_object_get_data(G_OBJECT(dialog), "rox-radios");
 	drop_box = g_object_get_data(G_OBJECT(dialog), "rox-dropbox");
 
 	g_return_if_fail(radios != NULL);
