@@ -1,8 +1,8 @@
 /*
- * $Id: pixmaps.c,v 1.116 2004/07/30 17:08:41 tal197 Exp $
+ * $Id: pixmaps.c,v 1.127 2005/12/29 10:14:34 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2003, the ROX-Filer team.
+ * Copyright (C) 2005, the ROX-Filer team.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -30,6 +30,7 @@
 
 #define PIXMAP_PURGE_TIME 1200
 #define PIXMAP_THUMB_SIZE  128
+#define PIXMAP_THUMB_TOO_OLD_TIME  5
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -57,6 +58,7 @@
 #include "type.h"
 
 GFSCache *pixmap_cache = NULL;
+GFSCache *desktop_icon_cache = NULL;
 
 static const char * bad_xpm[] = {
 "12 12 3 1",
@@ -83,6 +85,7 @@ MaskedPixmap *im_symlink;
 MaskedPixmap *im_unmounted;
 MaskedPixmap *im_mounted;
 MaskedPixmap *im_appdir;
+MaskedPixmap *im_xattr;
 
 MaskedPixmap *im_dirs;
 
@@ -110,13 +113,12 @@ static GtkIconSize mount_icon_size = -1;
 static void load_default_pixmaps(void);
 static gint purge(gpointer data);
 static MaskedPixmap *image_from_file(const char *path);
+static MaskedPixmap *image_from_desktop_file(const char *path);
 static MaskedPixmap *get_bad_image(void);
 static GdkPixbuf *scale_pixbuf_up(GdkPixbuf *src, int max_w, int max_h);
 static GdkPixbuf *get_thumbnail_for(const char *path);
 static void thumbnail_child_done(ChildThumbnail *info);
 static void child_create_thumbnail(const gchar *path);
-static GdkPixbuf *create_spotlight_pixbuf(GdkPixbuf *src, guint32 color,
-					  guchar alpha);
 static GList *thumbs_purge_cache(Option *option, xmlNode *node, guchar *label);
 static gchar *thumbnail_path(const gchar *path);
 static gchar *thumbnail_program(MIME_type *type);
@@ -133,6 +135,7 @@ void pixmaps_init(void)
 	gtk_widget_push_colormap(gdk_rgb_get_colormap());
 
 	pixmap_cache = g_fscache_new((GFSLoadFunc) image_from_file, NULL, NULL);
+	desktop_icon_cache = g_fscache_new((GFSLoadFunc) image_from_desktop_file, NULL, NULL);
 
 	g_timeout_add(10000, purge, NULL);
 
@@ -232,8 +235,6 @@ void pixmap_make_huge(MaskedPixmap *mp)
 		g_object_ref(mp->huge_pixbuf);
 	}
 
-	mp->huge_pixbuf_lit = create_spotlight_pixbuf(mp->huge_pixbuf,
-						      0x000099, 128);
 	mp->huge_width = gdk_pixbuf_get_width(mp->huge_pixbuf);
 	mp->huge_height = gdk_pixbuf_get_height(mp->huge_pixbuf);
 }
@@ -253,9 +254,6 @@ void pixmap_make_small(MaskedPixmap *mp)
 		g_object_ref(mp->sm_pixbuf);
 	}
 
-	mp->sm_pixbuf_lit = create_spotlight_pixbuf(mp->sm_pixbuf,
-						      0x000099, 128);
-
 	mp->sm_width = gdk_pixbuf_get_width(mp->sm_pixbuf);
 	mp->sm_height = gdk_pixbuf_get_height(mp->sm_pixbuf);
 }
@@ -315,7 +313,8 @@ void pixmap_background_thumb(const gchar *path, GFunc callback, gpointer data)
 			    info1.st_dev == info2.st_dev &&
 			    info1.st_ino == info2.st_ino)
 		{
-			pixbuf = gdk_pixbuf_new_from_file(path, NULL);
+			pixbuf = rox_pixbuf_new_from_file_at_scale(path,
+					PIXMAP_THUMB_SIZE, PIXMAP_THUMB_SIZE, TRUE, NULL);
 			if (!pixbuf)
 			{
 				g_fscache_insert(pixmap_cache,
@@ -334,6 +333,7 @@ void pixmap_background_thumb(const gchar *path, GFunc callback, gpointer data)
 		gdk_pixbuf_unref(pixbuf);
 		g_fscache_insert(pixmap_cache, path, image, TRUE);
 		callback(data, (gchar *) path);
+		g_object_unref(G_OBJECT(image));
 		return;
 	}
 
@@ -379,7 +379,7 @@ void pixmap_background_thumb(const gchar *path, GFunc callback, gpointer data)
 			if (item->flags & ITEM_FLAG_APPDIR)
 				thumb_prog = g_strconcat(thumb_prog, "/AppRun",
 						       NULL);
-						  
+
 			execl(thumb_prog, thumb_prog, path,
 			      thumbnail_path(path),
 			      g_strdup_printf("%d", PIXMAP_THUMB_SIZE),
@@ -431,8 +431,8 @@ static void save_thumbnail(const char *pathname, GdkPixbuf *full)
 
 	path = pathdup(pathname);
 	uri = g_filename_to_uri(path, NULL, NULL);
-	if(!uri)
-	        uri=g_strconcat("file://", path, NULL);
+	if (!uri)
+	        uri = g_strconcat("file://", path, NULL);
 	md5 = md5_hash(uri);
 	g_free(path);
 		
@@ -521,14 +521,15 @@ static gchar *thumbnail_program(MIME_type *type)
 		return NULL;
 
 	leaf = g_strconcat(type->media_type, "_", type->subtype, NULL);
-	path = choices_find_path_load(leaf, "MIME-thumb");
+	path = choices_find_xdg_path_load(leaf, "MIME-thumb", SITE);
 	g_free(leaf);
 	if (path)
 	{
 		return path;
 	}
 
-	path = choices_find_path_load(type->media_type, "MIME-thumb");
+	path = choices_find_xdg_path_load(type->media_type, "MIME-thumb",
+					  SITE);
 
 	return path;
 }
@@ -540,7 +541,8 @@ static void child_create_thumbnail(const gchar *path)
 {
 	GdkPixbuf *image;
 
-	image = gdk_pixbuf_new_from_file(path, NULL);
+	image = rox_pixbuf_new_from_file_at_scale(path,
+			PIXMAP_THUMB_SIZE, PIXMAP_THUMB_SIZE, TRUE, NULL);
 
 	if (image)
 		save_thumbnail(path, image);
@@ -583,6 +585,7 @@ static GdkPixbuf *get_thumbnail_for(const char *pathname)
 	char *thumb_path, *md5, *uri, *path;
 	const char *ssize, *smtime;
 	struct stat info;
+	time_t ttime, now;
 
 	path = pathdup(pathname);
 	uri = g_filename_to_uri(path, NULL, NULL);
@@ -611,7 +614,12 @@ static GdkPixbuf *get_thumbnail_for(const char *pathname)
 	if (mc_stat(path, &info) != 0)
 		goto err;
 
-	if (info.st_mtime != atol(smtime) || info.st_size != atol(ssize))
+	ttime=(time_t) atol(smtime);
+	time(&now);
+	if (info.st_mtime != ttime && now>ttime+PIXMAP_THUMB_TOO_OLD_TIME)
+		goto err;
+
+	if (info.st_size < atol(ssize))
 		goto err;
 
 	goto out;
@@ -650,6 +658,65 @@ static MaskedPixmap *image_from_file(const char *path)
 	return image;
 }
 
+/* Load this icon named by this .desktop file from the current theme.
+ * NULL on failure.
+ */
+static MaskedPixmap *image_from_desktop_file(const char *path)
+{
+	GError *error = NULL;
+	MaskedPixmap *image = NULL;
+	char *icon = NULL;
+
+	icon = get_value_from_desktop_file(path,
+					"Desktop Entry", "Icon", &error);
+	if (error)
+	{
+		g_warning("Failed to parse .desktop file '%s':\n%s",
+				path, error->message);
+		goto err;
+	}
+	if (!icon)
+		goto err;
+
+	if (icon[0] == '/')
+		image = image_from_file(icon);
+	else
+	{
+		GdkPixbuf *pixbuf;
+		int tmp_fd;
+		char *extension;
+
+		/* For some unknown reason, some icon names have extensions.
+		 * Remove them.
+		 */
+		extension = strrchr(icon, '.');
+		if (extension && strcmp(extension, ".png") == 0)
+		{
+			*extension = '\0';
+		}
+
+		/* SVG reader is very noisy, so redirect stderr to stdout */
+		tmp_fd = dup(2);
+		dup2(1, 2);
+		pixbuf = gtk_icon_theme_load_icon(icon_theme, icon, HUGE_WIDTH,
+							0, NULL);
+		dup2(tmp_fd, 2);
+		close(tmp_fd);
+
+		if (pixbuf == NULL)
+			goto err;	/* Might just not be in the theme */
+
+		image = masked_pixmap_new(pixbuf);
+		g_object_unref(pixbuf);
+	}
+err:
+	if (error != NULL)
+		g_error_free(error);
+	if (icon != NULL)
+		g_free(icon);
+	return image;
+}
+
 /* Scale src down to fit in max_w, max_h and return the new pixbuf.
  * If src is small enough, then ref it and return that.
  */
@@ -748,33 +815,17 @@ static void masked_pixmap_finialize(GObject *object)
 		g_object_unref(mp->huge_pixbuf);
 		mp->huge_pixbuf = NULL;
 	}
-	if (mp->huge_pixbuf_lit)
-	{
-		g_object_unref(mp->huge_pixbuf_lit);
-		mp->huge_pixbuf_lit = NULL;
-	}
-
 	if (mp->pixbuf)
 	{
 		g_object_unref(mp->pixbuf);
 		mp->pixbuf = NULL;
 	}
-	if (mp->pixbuf_lit)
-	{
-		g_object_unref(mp->pixbuf_lit);
-		mp->pixbuf_lit = NULL;
-	}
 
 	if (mp->sm_pixbuf)
 	{
 		g_object_unref(mp->sm_pixbuf);
 		mp->sm_pixbuf = NULL;
 	}
-	if (mp->sm_pixbuf_lit)
-	{
-		g_object_unref(mp->sm_pixbuf_lit);
-		mp->sm_pixbuf_lit = NULL;
-	}
 
 	G_OBJECT_CLASS(parent_class)->finalize(object);
 }
@@ -795,17 +846,14 @@ static void masked_pixmap_init(GTypeInstance *object, gpointer gclass)
 	mp->src_pixbuf = NULL;
 
 	mp->huge_pixbuf = NULL;
-	mp->huge_pixbuf_lit = NULL;
 	mp->huge_width = -1;
 	mp->huge_height = -1;
 
 	mp->pixbuf = NULL;
-	mp->pixbuf_lit = NULL;
 	mp->width = -1;
 	mp->height = -1;
 
 	mp->sm_pixbuf = NULL;
-	mp->sm_pixbuf_lit = NULL;
 	mp->sm_width = -1;
 	mp->sm_height = -1;
 }
@@ -854,75 +902,12 @@ MaskedPixmap *masked_pixmap_new(GdkPixbuf *full_size)
 	mp->src_pixbuf = src_pixbuf;
 
 	mp->pixbuf = normal_pixbuf;
-	mp->pixbuf_lit = create_spotlight_pixbuf(normal_pixbuf, 0x000099, 128);
 	mp->width = gdk_pixbuf_get_width(normal_pixbuf);
 	mp->height = gdk_pixbuf_get_height(normal_pixbuf);
 
 	return mp;
 }
 
-/* Stolen from eel...and modified to colourize the pixbuf.
- * 'alpha' is the transparency of 'color' (0xRRGGBB):
- * 0 = fully opaque, 255 = fully transparent.
- */
-static GdkPixbuf *create_spotlight_pixbuf(GdkPixbuf *src,
-					  guint32 color,
-					  guchar alpha)
-{
-	GdkPixbuf *dest;
-	int i, j;
-	int width, height, has_alpha, src_row_stride, dst_row_stride;
-	guchar *target_pixels, *original_pixels;
-	guchar *pixsrc, *pixdest;
-	guchar r, g, b;
-	gint n_channels;
-
-	n_channels = gdk_pixbuf_get_n_channels(src);
-	has_alpha = gdk_pixbuf_get_has_alpha(src);
-	width = gdk_pixbuf_get_width(src);
-	height = gdk_pixbuf_get_height(src);
-
-	g_return_val_if_fail(gdk_pixbuf_get_colorspace(src) ==
-			     GDK_COLORSPACE_RGB, NULL);
-	g_return_val_if_fail((!has_alpha && n_channels == 3) ||
-			     (has_alpha && n_channels == 4), NULL);
-	g_return_val_if_fail(gdk_pixbuf_get_bits_per_sample(src) == 8, NULL);
-
-	dest = gdk_pixbuf_new(gdk_pixbuf_get_colorspace(src), has_alpha,
-			       gdk_pixbuf_get_bits_per_sample(src),
-			       width, height);
-	
-	dst_row_stride = gdk_pixbuf_get_rowstride(dest);
-	src_row_stride = gdk_pixbuf_get_rowstride(src);
-	target_pixels = gdk_pixbuf_get_pixels(dest);
-	original_pixels = gdk_pixbuf_get_pixels(src);
-
-	r = (color & 0xff0000) >> 16;
-	g = (color & 0xff00) >> 8;
-	b = color & 0xff;
-
-	for (i = 0; i < height; i++)
-	{
-		gint tmp;
-
-		pixdest = target_pixels + i * dst_row_stride;
-		pixsrc = original_pixels + i * src_row_stride;
-		for (j = 0; j < width; j++)
-		{
-			tmp = (*pixsrc++ * alpha + r * (255 - alpha)) / 255;
-			*pixdest++ = (guchar) MIN(255, tmp);
-			tmp = (*pixsrc++ * alpha + g * (255 - alpha)) / 255;
-			*pixdest++ = (guchar) MIN(255, tmp);
-			tmp = (*pixsrc++ * alpha + b * (255 - alpha)) / 255;
-			*pixdest++ = (guchar) MIN(255, tmp);
-			if (has_alpha)
-				*pixdest++ = *pixsrc++;
-		}
-	}
-
-	return dest;
-}
-
 /* Load all the standard pixmaps. Also sets the default window icon. */
 static void load_default_pixmaps(void)
 {
@@ -938,6 +923,7 @@ static void load_default_pixmaps(void)
 	im_unmounted = mp_from_stock(ROX_STOCK_MOUNT, mount_icon_size);
 	im_mounted = mp_from_stock(ROX_STOCK_MOUNTED, mount_icon_size);
 	im_appdir = load_pixmap("application");
+	im_xattr = load_pixmap("rox-xattr");
 
 	im_dirs = load_pixmap("dirs");
 
