@@ -1,8 +1,8 @@
 /*
- * $Id: support.c,v 1.112 2004/08/01 09:45:53 tal197 Exp $
+ * $Id: support.c,v 1.119 2005/11/13 12:27:49 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2003, the ROX-Filer team.
+ * Copyright (C) 2005, the ROX-Filer team.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -38,6 +38,7 @@
 #include <unistd.h>
 #include <libxml/parser.h>
 #include <math.h>
+#include <sys/mman.h>
 
 #include "global.h"
 
@@ -474,28 +475,34 @@ static gboolean is_local_address(char *address)
  *	//host/path
  *	file://host/path
  */
-char *get_local_path(const char *uri)
+char *get_local_path(const EscapedPath *escaped_uri)
 {
+	const char *uri = (char *) escaped_uri;
+
 	if (*uri == '/')
 	{
 		char    *path, *uri_host;
 
 		/* Just a local path - no host part */
 		if (uri[1] != '/')
-			return unescape_uri(uri);
+			return unescape_uri((EscapedPath *) uri);
 		                    
 		path = strchr(uri + 2, '/');
 		if (!path)
 			return NULL;	    /* //something */
 
 		if (path - uri == 2)
-			return unescape_uri(path);	/* ///path */
+		{
+			/* ///path */
+			return unescape_uri((EscapedPath *) path);
+		}
 		
 		uri_host = g_strndup(uri + 2, path - uri - 2);
 		if (is_local_address(uri_host))
 		{
 			g_free(uri_host);
-			return unescape_uri(path);	/* //myhost/path */
+			/* //myhost/path */
+			return unescape_uri((EscapedPath *) path);
 		}
 		g_free(uri_host);
 
@@ -509,7 +516,7 @@ char *get_local_path(const char *uri)
 		uri += 5;
 
 		if (*uri == '/')
-			return get_local_path(uri);
+			return get_local_path((EscapedPath *) uri);
 
 		return NULL;
 	}
@@ -1169,10 +1176,15 @@ CollateKey *collate_key_new(const guchar *name)
 
 	for (i = name; *i; i = g_utf8_next_char(i))
 	{
+		gunichar first_char;
+
 		/* We're in a (possibly blank) text section starting at 'name'.
 		 * Find the end of it (the next digit, or end of string).
+		 * Note: g_ascii_isdigit takes char, not unichar, while
+		 * g_unicode_isdigit returns true for non ASCII digits.
 		 */
-		if (g_unichar_isdigit(g_utf8_get_char(i)))
+		first_char = g_utf8_get_char(i);
+		if (first_char >= '0' && first_char <= '9')
 		{
 			char *endp;
 			
@@ -1265,9 +1277,9 @@ gboolean file_exists(const char *path)
 }
 
 /* Escape path for future use in URI */
-gchar *escape_uri_path(const char *path)
+EscapedPath *escape_uri_path(const char *path)
 {
-	const char *safe = "-_./"; /* Plus isalnum() */
+	const char *safe = ":-_./"; /* Plus isalnum() */
 	const guchar *s;
 	gchar *ans;
 	GString *str;
@@ -1285,28 +1297,29 @@ gchar *escape_uri_path(const char *path)
 	ans = str->str;
 	g_string_free(str, FALSE);
 
-	return ans;
+	return (EscapedPath *) ans;
 }
 
-gchar *encode_path_as_uri(const guchar *path)
+EscapedPath *encode_path_as_uri(const guchar *path)
 {
-	gchar *tpath = escape_uri_path(path);
+	gchar *tpath = (gchar *) escape_uri_path(path);
 	gchar *uri;
 
 	uri = g_strconcat("file://", our_host_name_for_dnd(), tpath, NULL);
 	g_free(tpath);
 
-	return uri;
+	return (EscapedPath *) uri;
 }
 
-gchar *unescape_uri(const char *uri)
+gchar *unescape_uri(const EscapedPath *uri)
 {
+	const char *uri_string = (char *) uri;
 	const gchar *s;
 	gchar *d;
 	gchar *tmp;
 	
-	tmp = g_malloc(strlen(uri) + 1);
-	for (s = uri, d = tmp; *s; s++, d++)
+	tmp = g_malloc(strlen(uri_string) + 1);
+	for (s = uri_string, d = tmp; *s; s++, d++)
 	{
 		/*printf("%s\n", s);*/
 		if (*s == '%' && g_ascii_isxdigit(s[1]) &&
@@ -1401,3 +1414,181 @@ int stat_with_timeout(const char *path, struct stat *info)
 
 	return retval;
 }
+
+/* From glib. */
+static gchar *my_strchrnul(const gchar *str, gchar c)
+{
+	gchar *p = (gchar*) str;
+	while (*p && (*p != c))
+		++p;
+
+	return p;
+}
+
+/* Based on code from glib. */
+gboolean available_in_path(const char *file)
+{
+	const gchar *path, *p;
+	gchar *name, *freeme;
+	size_t len;
+	size_t pathlen;
+	gboolean found = FALSE;
+
+	path = g_getenv("PATH");
+	if (path == NULL)
+		path = "/bin:/usr/bin:.";
+
+	len = strlen(file) + 1;
+	pathlen = strlen(path);
+	freeme = name = g_malloc(pathlen + len + 1);
+
+	/* Copy the file name at the top, including '\0'  */
+	memcpy(name + pathlen + 1, file, len);
+	name = name + pathlen;
+	/* And add the slash before the filename  */
+	*name = '/';
+
+	p = path;
+	do
+	{
+		char *startp;
+
+		path = p;
+		p = my_strchrnul(path, ':');
+
+		if (p == path)
+			/* Two adjacent colons, or a colon at the beginning or
+			 * the end of `PATH' means to search the current
+			 * directory.
+			 */
+			startp = name + 1;
+		else
+			startp = memcpy (name - (p - path), path, p - path);
+
+		/* Try to execute this name.  If it works, execv will not
+		 * return.
+		 */
+		if (access(startp, X_OK) == 0)
+			found = TRUE;
+	} while (!found && *p++ != '\0');
+
+	g_free(freeme);
+
+	return found;
+}
+
+static char *get_value_from_desktop_data(const char *data, size_t length,
+					 const char *section, const char *key,
+					 GError **error)
+{
+	int section_length, key_length;
+	const char *last_possible_key_start;
+	const char *last_possible_section_start;
+	const char *next;
+
+	section_length = strlen(section);
+	last_possible_section_start = data + length - section_length - 3;
+
+	for (next = data; next <= last_possible_section_start; next++)
+	{
+		if (next[0] != '[' || (next > data && next[-1] != '\n'))
+			continue;
+		next += 1;
+		if (next[section_length] != ']' ||
+		    next[section_length + 1] != '\n')
+			continue;
+		if (strncmp(next, section, section_length) != 0)
+			continue;
+		next += section_length + 1;	/* Newline */
+		goto find_key;
+	}
+	return NULL;
+
+find_key:
+	key_length = strlen(key);
+	last_possible_key_start = data + length - key_length - 3;
+	for (;
+	     next && next < last_possible_key_start;
+	     next = memchr(next + 1, '\n', last_possible_key_start - next))
+	{
+		const char *nl;
+
+		next++;		/* Skip newline */
+
+		if (next[0] == '[')
+			break;	/* End of section */
+
+		if (strncmp(next, key, key_length) != 0)
+			continue;
+		next += key_length;
+		while (next[0] == ' ' || next[0] == '\t')
+			next++;
+
+		/* Note: if we had GLib 2.6, we could use
+		 * g_get_language_names() to get translations here.
+		 */
+		if (next[0] != '=')
+			continue;
+		next++;
+		while (next[0] == ' ' || next[0] == '\t')
+			next++;
+		nl = memchr(next, '\n', data + length - next);
+		if (!nl)
+			break;
+		return g_strndup(next, nl - next);
+	}
+
+	return NULL;
+}
+
+/* Load .desktop file 'path' and return the value of the named key.
+ * Sets error if the desktop file cannot be parsed.
+ * Returns NULL (but does not set error) if the key is not present.
+ * Returned string must be g_free()d.
+ */
+char *get_value_from_desktop_file(const char *path,
+				  const char *section,
+				  const char *key,
+				  GError **error)
+{
+	char *value = NULL;
+	struct stat info;
+	int fd = -1;
+	void *start = NULL;
+
+	fd = open(path, O_RDONLY);
+	if (fd == -1 || fstat(fd, &info) != 0)
+	{
+                g_set_error(error,
+                             G_FILE_ERROR,
+                             g_file_error_from_errno(errno),
+                             _("Failed to open and stat file '%s': %s"),
+                             path,
+                             g_strerror(errno));
+		goto err;
+	}
+	start = mmap(NULL, info.st_size, PROT_READ, MAP_SHARED, fd, 0);
+	if (start == MAP_FAILED)
+	{
+                g_set_error(error,
+                             G_FILE_ERROR,
+                             g_file_error_from_errno(errno),
+                             _("Failed to mmap file '%s': %s"),
+                             path,
+                             g_strerror(errno));
+		goto err;
+	}
+
+	value = get_value_from_desktop_data((const char *) start,
+					    info.st_size,
+					    section, key,
+					    error);
+err:
+	if (fd != -1)
+		close(fd);
+
+	if (start != NULL && start != MAP_FAILED)
+		munmap(start, info.st_size);
+
+	return value;
+}
