@@ -1,8 +1,8 @@
 /*
- * $Id: toolbar.c,v 1.76 2004/05/14 10:43:59 tal197 Exp $
+ * $Id: toolbar.c,v 1.83 2005/12/15 18:39:54 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2003, the ROX-Filer team.
+ * Copyright (C) 2005, the ROX-Filer team.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -59,8 +59,8 @@ struct _Tool {
 };
 
 Option o_toolbar, o_toolbar_info, o_toolbar_disable;
+Option o_toolbar_min_width;
 
-static GtkTooltips *tooltips = NULL;
 static FilerWindow *filer_window_being_counted;
 
 /* TRUE if the button presses (or released) should open a new window,
@@ -172,10 +172,9 @@ void toolbar_init(void)
 	option_add_int(&o_toolbar_info, "toolbar_show_info", 1);
 	option_add_string(&o_toolbar_disable, "toolbar_disable",
 					GTK_STOCK_CLOSE);
+	option_add_int(&o_toolbar_min_width, "toolbar_min_width", 1);
 	option_add_notify(option_notify);
 	
-	tooltips = gtk_tooltips_new();
-
 	option_register_widget("tool-options", build_tool_options);
 }
 
@@ -185,6 +184,8 @@ void toolbar_update_info(FilerWindow *filer_window)
 	ViewIface	*view;
 	int		n_selected;
 
+	g_return_if_fail(filer_window != NULL);
+
 	if (o_toolbar.int_value == TOOLBAR_NONE || !o_toolbar_info.int_value)
 		return;		/* Not showing info */
 
@@ -242,8 +243,11 @@ void toolbar_update_info(FilerWindow *filer_window)
 		view_get_iter(filer_window->view, &iter, VIEW_ITER_SELECTED);
 
 		while ((item = iter.next(&iter)))
-			if (item->base_type != TYPE_DIRECTORY)
+		{
+			if (item->base_type != TYPE_DIRECTORY &&
+			    item->base_type != TYPE_UNKNOWN)
 				size += (double) item->size;
+		}
 
 		label = g_strdup_printf(_("%u selected (%s)"),
 				n_selected, format_double_size(size));
@@ -295,6 +299,7 @@ static void toolbar_help_clicked(GtkWidget *widget, FilerWindow *filer_window)
 		menu_rox_help(NULL, HELP_MANUAL, NULL);
 	else
 		filer_opendir(make_path(app_dir, "Help"), NULL, NULL);
+	gdk_event_free(event);
 }
 
 static void toolbar_refresh_clicked(GtkWidget *widget,
@@ -310,6 +315,7 @@ static void toolbar_refresh_clicked(GtkWidget *widget,
 	}
 	else
 		filer_refresh(filer_window);
+	gdk_event_free(event);
 }
 
 static void toolbar_home_clicked(GtkWidget *widget, FilerWindow *filer_window)
@@ -323,6 +329,7 @@ static void toolbar_home_clicked(GtkWidget *widget, FilerWindow *filer_window)
 	}
 	else
 		filer_change_to(filer_window, home_dir, NULL);
+	gdk_event_free(event);
 }
 
 static void toolbar_bookmarks_clicked(GtkWidget *widget,
@@ -343,6 +350,7 @@ static void toolbar_bookmarks_clicked(GtkWidget *widget,
 	{
 		bookmarks_edit();
 	}
+	gdk_event_free(event);
 }
 
 static void toolbar_close_clicked(GtkWidget *widget, FilerWindow *filer_window)
@@ -359,6 +367,7 @@ static void toolbar_close_clicked(GtkWidget *widget, FilerWindow *filer_window)
 	}
 	else if (!filer_window_delete(filer_window->window, NULL, filer_window))
 		gtk_widget_destroy(filer_window->window);
+	gdk_event_free(event);
 }
 
 static void toolbar_up_clicked(GtkWidget *widget, FilerWindow *filer_window)
@@ -372,6 +381,7 @@ static void toolbar_up_clicked(GtkWidget *widget, FilerWindow *filer_window)
 	}
 	else
 		change_to_parent(filer_window);
+	gdk_event_free(event);
 }
 
 static void toolbar_autosize_clicked(GtkWidget *widget, FilerWindow *filer_window)
@@ -379,11 +389,12 @@ static void toolbar_autosize_clicked(GtkWidget *widget, FilerWindow *filer_windo
 	GdkEventButton	*bev;
 
 	bev = (GdkEventButton *) gtk_get_current_event();
-	if (bev->type != GDK_BUTTON_RELEASE)
-		return;
-
-	display_set_layout(filer_window, AUTO_SIZE_ICONS, filer_window->details_type,
-			   TRUE);
+	if (bev->type == GDK_BUTTON_RELEASE)
+	{
+		display_set_layout(filer_window, AUTO_SIZE_ICONS, filer_window->details_type,
+				TRUE);
+	}
+	gdk_event_free((GdkEvent *) bev);
 }
 
 static void toolbar_size_clicked(GtkWidget *widget, FilerWindow *filer_window)
@@ -391,10 +402,9 @@ static void toolbar_size_clicked(GtkWidget *widget, FilerWindow *filer_window)
 	GdkEventButton	*bev;
 
 	bev = (GdkEventButton *) gtk_get_current_event();
-	if (bev->type != GDK_BUTTON_RELEASE)
-		return;
-
-	display_change_size(filer_window, bev->button == 1);
+	if (bev->type == GDK_BUTTON_RELEASE)
+		display_change_size(filer_window, bev->button == 1);
+	gdk_event_free((GdkEvent *) bev);
 }
 
 static void toolbar_sort_clicked(GtkWidget *widget,
@@ -417,6 +427,7 @@ static void toolbar_sort_clicked(GtkWidget *widget,
 
 	bev = (GdkEventButton *) gtk_get_current_event();
 	adjust = (bev->button != 1) && bev->type == GDK_BUTTON_RELEASE;
+	gdk_event_free((GdkEvent *) bev);
 
 	current = -1;
 	dir = filer_window->sort_order;
@@ -485,6 +496,7 @@ static void toolbar_select_clicked(GtkWidget *widget, FilerWindow *filer_window)
 				       filer_window->view);
 	}
 	filer_window->temp_item_selected = FALSE;
+	gdk_event_free(event);
 }
 
 /* If filer_window is NULL, the toolbar is for the options window */
@@ -493,12 +505,10 @@ static GtkWidget *create_toolbar(FilerWindow *filer_window)
 	GtkWidget	*bar;
 	GtkWidget	*b;
 	int		i;
+	int		width;
 
 	bar = gtk_toolbar_new();
 
-	if (filer_window)
-		gtk_widget_set_size_request(bar, 100, -1);
-
 	if (o_toolbar.int_value == TOOLBAR_NORMAL || !filer_window)
 		gtk_toolbar_set_style(GTK_TOOLBAR(bar), GTK_TOOLBAR_ICONS);
 	else if (o_toolbar.int_value == TOOLBAR_HORIZONTAL)
@@ -506,20 +516,36 @@ static GtkWidget *create_toolbar(FilerWindow *filer_window)
 	else
 		gtk_toolbar_set_style(GTK_TOOLBAR(bar), GTK_TOOLBAR_BOTH);
 
+	width=0;
 	for (i = 0; i < sizeof(all_tools) / sizeof(*all_tools); i++)
 	{
 		Tool	*tool = &all_tools[i];
+		GtkRequisition req;
 
 		if (filer_window && !tool->enabled)
 			continue;
 
 		b = add_button(bar, tool, filer_window);
+		
+		gtk_widget_size_request(b, &req);
+		width+=req.width;
+		
 		if (filer_window && tool->drop_action != DROP_NONE)
 			handle_drops(filer_window, b, tool->drop_action);
 	}
 
 	if (filer_window)
 	{
+		if(o_toolbar_min_width.int_value)
+		{
+			/* Make the toolbar wide enough for all icons to be
+			   seen, plus a little for the (start of the) text
+			   label */
+			gtk_widget_set_size_request(bar, width+32, -1);
+		} else {
+			gtk_widget_set_size_request(bar, 100, -1);
+		}
+		
 		filer_window->toolbar_text = gtk_label_new("");
 		gtk_misc_set_alignment(GTK_MISC(filer_window->toolbar_text),
 					0, 0.5);
