@@ -1,5 +1,5 @@
 /*
- * $Id: type.c,v 1.161 2005/01/16 16:48:03 tal197 Exp $
+ * $Id: type.c,v 1.175 2005/12/18 14:07:02 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
  * Copyright (C) 2005, the ROX-Filer team.
@@ -58,7 +58,9 @@
 #include "action.h"		/* (for action_chmod) */
 #include "xml.h"
 #include "dropbox.h"
-#include "gtkicontheme.h"
+#include "xdgmime.h"
+#include "xtypes.h"
+#include "run.h"
 
 #define TYPE_NS "http://www.freedesktop.org/standards/shared-mime-info"
 enum {SET_MEDIA, SET_TYPE};
@@ -85,7 +87,6 @@ static Option o_type_colours[NUM_TYPE_COLOURS];
 static GdkColor	type_colours[NUM_TYPE_COLOURS];
 
 /* Static prototypes */
-static void load_mime_types(void);
 static void alloc_type_colours(void);
 static void options_changed(void);
 static char *get_action_save_path(GtkWidget *dialog);
@@ -94,26 +95,6 @@ static gboolean remove_handler_with_confirm(const guchar *path);
 static void set_icon_theme(void);
 static GList *build_icon_theme(Option *option, xmlNode *node, guchar *label);
 
-/* When working out the type for a file, this hash table is checked
- * first...
- */
-static GHashTable *literal_hash = NULL;	/* name -> MIME-type */
-
-/* Maps extensions to MIME_types (eg 'png'-> MIME_type *).
- * Extensions may contain dots; 'tar.gz' matches '*.tar.gz', etc.
- * The hash table is consulted from each dot in the string in turn
- * (First .ps.gz, then try .gz)
- */
-static GHashTable *extension_hash = NULL;
-
-/* The first pattern in the list which matches is used */
-typedef struct pattern {
-	gint len;		/* Used for sorting */
-	gchar *glob;
-	MIME_type *type;
-} Pattern;
-static GPtrArray *glob_patterns = NULL;	/* [Pattern] */
-
 /* Hash of all allocated MIME types, indexed by "media/subtype".
  * MIME_type structs are never freed; this table prevents memory leaks
  * when rereading the config files.
@@ -129,21 +110,23 @@ MIME_type *inode_socket;
 MIME_type *inode_block_dev;
 MIME_type *inode_char_dev;
 MIME_type *application_executable;
+MIME_type *application_octet_stream;
+MIME_type *application_x_shellscript;
+MIME_type *application_x_desktop;
 MIME_type *inode_unknown;
 MIME_type *inode_door;
 
 static Option o_display_colour_types;
 static Option o_icon_theme;
 
-static RoxIconTheme *icon_theme = NULL;
+GtkIconTheme *icon_theme = NULL;
 
 void type_init(void)
 {
 	int	    i;
 
-	icon_theme = rox_icon_theme_new();
+	icon_theme = gtk_icon_theme_new();
 	
-	extension_hash = g_hash_table_new(g_str_hash, g_str_equal);
 	type_hash = g_hash_table_new(g_str_hash, g_str_equal);
 
 	text_plain = get_mime_type("text/plain", TRUE);
@@ -154,11 +137,13 @@ void type_init(void)
 	inode_block_dev = get_mime_type("inode/blockdevice", TRUE);
 	inode_char_dev = get_mime_type("inode/chardevice", TRUE);
 	application_executable = get_mime_type("application/x-executable", TRUE);
+	application_octet_stream = get_mime_type("application/octet-stream", TRUE);
+	application_x_shellscript = get_mime_type("application/x-shellscript", TRUE);
+	application_x_desktop = get_mime_type("application/x-desktop", TRUE);
+	application_x_desktop->executable = TRUE;
 	inode_unknown = get_mime_type("inode/unknown", TRUE);
 	inode_door = get_mime_type("inode/door", TRUE);
 
-	load_mime_types();
-
 	option_add_string(&o_icon_theme, "icon_theme", "ROX");
 	option_add_int(&o_display_colour_types, "display_colour_types", TRUE);
 	option_register_widget("icon-theme-chooser", build_icon_theme);
@@ -179,8 +164,11 @@ void type_init(void)
  */
 void reread_mime_files(void)
 {
-	rox_icon_theme_rescan_if_needed(icon_theme);
-	load_mime_types();
+	gtk_icon_theme_rescan_if_needed(icon_theme);
+
+	xdg_mime_shutdown();
+
+	filer_update_all();
 }
 
 /* Returns the MIME_type structure for the given type name. It is looked
@@ -207,28 +195,14 @@ static MIME_type *get_mime_type(const gchar *type_name, gboolean can_create)
 	mtype->image = NULL;
 	mtype->comment = NULL;
 
+	mtype->executable = xdg_mime_mime_type_subclass(type_name,
+						"application/x-executable");
+
 	g_hash_table_insert(type_hash, g_strdup(type_name), mtype);
 
 	return mtype;
 }
 
-/* NULL if we don't know / don't support contents checking */
-MIME_type *mime_type_from_contents(const char *path)
-{
-	MIME_type *type = NULL;
-#ifdef HAVE_GNOME_VFS
-	char *type_name;
-
-	type_name = gnome_vfs_get_mime_type(path);
-	if (type_name)
-	{
-		type = get_mime_type(type_name, TRUE);
-		free(type_name);
-	}
-#endif
-	return type;
-}
-
 const char *basetype_name(DirItem *item)
 {
 	if (item->flags & ITEM_FLAG_SYMLINK)
@@ -307,74 +281,47 @@ MIME_type *type_get_type(const guchar *path)
 }
 
 /* Returns a pointer to the MIME-type.
+ *
+ * Tries all enabled methods:
+ * - Look for extended attribute
+ * - If no attribute, check file name
+ * - If no name rule, check contents
+ *
  * NULL if we can't think of anything.
  */
 MIME_type *type_from_path(const char *path)
 {
-	const char *ext, *dot, *leafname;
-	char *lower;
-	MIME_type *type = NULL;
-	int	i;
+	MIME_type *mime_type = NULL;
+	const char *type_name;
 
-	leafname = g_basename(path);
+	/* Check for extended attribute first */
+	mime_type = xtype_get(path);
+	if (mime_type)
+		return mime_type;
 
-	type = g_hash_table_lookup(literal_hash, leafname);
-	if (type)
-		return type;
-	lower = g_utf8_strdown(leafname, -1);
-	type = g_hash_table_lookup(literal_hash, lower);
-	if (type)
-		goto out;
+	/* Try name and contents next */
+	type_name = xdg_mime_get_mime_type_for_file(path);
+	if (type_name)
+		return get_mime_type(type_name, TRUE);
 
-	ext = leafname;
-
-	while ((dot = strchr(ext, '.')))
-	{
-		ext = dot + 1;
-
-		type = g_hash_table_lookup(extension_hash, ext);
-
-		if (type)
-			goto out;
-
-		type = g_hash_table_lookup(extension_hash,
-					lower + (ext - leafname));
-		if (type)
-			goto out;
-	}
-
-	for (i = 0; i < glob_patterns->len; i++)
-	{
-		Pattern *p = glob_patterns->pdata[i];
-
-		if (fnmatch(p->glob, leafname, 0) == 0 ||
-		    fnmatch(p->glob, lower, 0) == 0)
-		{
-			type = p->type;
-			goto out;
-		}
-	}
-
-out:
-	g_free(lower);
-
-	return type;
+	return NULL;
 }
 
 /* Returns the file/dir in Choices for handling this type.
  * NULL if there isn't one. g_free() the result.
  */
-static char *handler_for(MIME_type *type)
+char *handler_for(MIME_type *type)
 {
 	char	*type_name;
 	char	*open;
 
 	type_name = g_strconcat(type->media_type, "_", type->subtype, NULL);
-	open = choices_find_path_load(type_name, "MIME-types");
+	open = choices_find_xdg_path_load(type_name, "MIME-types", SITE);
 	g_free(type_name);
 
 	if (!open)
-		open = choices_find_path_load(type->media_type, "MIME-types");
+		open = choices_find_xdg_path_load(type->media_type,
+						  "MIME-types", SITE);
 
 	return open;
 }
@@ -386,66 +333,6 @@ MIME_type *mime_type_lookup(const char *type)
 
 /*			Actions for types 			*/
 
-gboolean type_open(const char *path, MIME_type *type)
-{
-	gchar *argv[] = {NULL, NULL, NULL};
-	char		*open;
-	gboolean	retval;
-	struct stat	info;
-
-	argv[1] = (char *) path;
-
-	open = handler_for(type);
-	if (!open)
-		return FALSE;
-
-	if (stat(open, &info))
-	{
-		report_error("stat(%s): %s", open, g_strerror(errno));
-		g_free(open);
-		return FALSE;
-	}
-
-	if (info.st_mode & S_IWOTH)
-	{
-		gchar *choices_dir;
-		GList *paths;
-
-		report_error(_("Executable '%s' is world-writeable! Refusing "
-			"to run. Please change the permissions now (this "
-			"problem may have been caused by a bug in earlier "
-			"versions of the filer).\n\n"
-			"Having (non-symlink) run actions world-writeable "
-			"means that other people who use your computer can "
-			"replace your run actions with malicious versions.\n\n"
-			"If you trust everyone who could write to these files "
-			"then you needn't worry. Otherwise, you should check, "
-			"or even just delete, all the existing run actions."),
-			open);
-		choices_dir = g_path_get_dirname(open);
-		paths = g_list_append(NULL, choices_dir);
-		action_chmod(paths, TRUE, _("go-w (Fix security problem)"));
-		g_free(choices_dir);
-		g_list_free(paths);
-		g_free(open);
-		return TRUE;
-	}
-
-	if (S_ISDIR(info.st_mode))
-		argv[0] = g_strconcat(open, "/AppRun", NULL);
-	else
-		argv[0] = open;
-
-	retval = rox_spawn(home_dir, (const gchar **) argv) != 0;
-
-	if (argv[0] != open)
-		g_free(argv[0]);
-
-	g_free(open);
-	
-	return retval;
-}
-
 /* Return the image for this type, loading it if needed.
  * Places to check are: (eg type="text_plain", base="text")
  * 1. <Choices>/MIME-icons/base_subtype
@@ -483,7 +370,7 @@ MaskedPixmap *type_to_icon(MIME_type *type)
 
 	type_name = g_strconcat(type->media_type, "_", type->subtype,
 				".png", NULL);
-	path = choices_find_path_load(type_name, "MIME-icons");
+	path = choices_find_xdg_path_load(type_name, "MIME-icons", SITE);
 	g_free(type_name);
 	if (path)
 	{
@@ -496,15 +383,18 @@ MaskedPixmap *type_to_icon(MIME_type *type)
 
 	type_name = g_strconcat("mime-", type->media_type, ":",
 				type->subtype, NULL);
-	full = rox_icon_theme_load_icon(icon_theme, type_name, HUGE_HEIGHT,
+	full = gtk_icon_theme_load_icon(icon_theme, type_name, HUGE_HEIGHT,
 						0, NULL);
 	g_free(type_name);
 	if (!full)
 	{
 		/* Ugly hack... try for a GNOME icon */
-		type_name = g_strconcat("gnome-mime-", type->media_type,
-				"-", type->subtype, NULL);
-		full = rox_icon_theme_load_icon(icon_theme,
+                if (type == inode_directory)
+			type_name = g_strdup("gnome-fs-directory");
+		else
+			type_name = g_strconcat("gnome-mime-", type->media_type,
+					"-", type->subtype, NULL);
+		full = gtk_icon_theme_load_icon(icon_theme,
 						type_name,
 						HUGE_HEIGHT, 0, NULL);
 		g_free(type_name);
@@ -513,11 +403,21 @@ MaskedPixmap *type_to_icon(MIME_type *type)
 	{
 		/* Try for a media type */
 		type_name = g_strconcat("mime-", type->media_type, NULL);
-		full = rox_icon_theme_load_icon(icon_theme,
+		full = gtk_icon_theme_load_icon(icon_theme,
 						type_name,
 						HUGE_HEIGHT, 0, NULL);
 		g_free(type_name);
 	}
+	if (!full)
+	{
+		/* Ugly hack... try for a GNOME default media icon */
+		type_name = g_strconcat("gnome-mime-", type->media_type, NULL);
+
+		full = gtk_icon_theme_load_icon(icon_theme,
+				type_name,
+				HUGE_HEIGHT, 0, NULL);
+		g_free(type_name);
+	}
 	if (full)
 	{
 		type->image = masked_pixmap_new(full);
@@ -645,14 +545,16 @@ static guchar *handler_for_radios(GObject *dialog)
 	switch (radios_get_value(radios))
 	{
 		case SET_MEDIA:
-			return choices_find_path_load(type->media_type,
-							 "MIME-types");
+			return choices_find_xdg_path_load(type->media_type,
+							  "MIME-types", SITE);
 		case SET_TYPE:
 		{
 			gchar *tmp, *handler;
 			tmp = g_strconcat(type->media_type, "_",
 					  type->subtype, NULL);
-			handler = choices_find_path_load(tmp, "MIME-types");
+			handler = choices_find_xdg_path_load(tmp,
+							     "MIME-types",
+							     SITE);
 			g_free(tmp);
 			return handler;
 		}
@@ -662,7 +564,8 @@ static guchar *handler_for_radios(GObject *dialog)
 	}
 }
 
-static void run_action_update(gpointer data)
+/* (radios can be NULL if called from clear_run_action) */
+static void run_action_update(Radios *radios, gpointer data)
 {
 	guchar *handler;
 	DropBox *drop_box;
@@ -698,7 +601,7 @@ static void clear_run_action(GtkWidget *drop_box, GtkWidget *dialog)
 	if (handler)
 		remove_handler_with_confirm(handler);
 
-	run_action_update(dialog);
+	run_action_update(NULL, dialog);
 }
 
 /* Called when a URI list is dropped onto the box in the Set Run Action
@@ -713,7 +616,7 @@ static void drag_app_dropped(GtkWidget	*drop_box,
 
 	item = diritem_new("");
 	diritem_restat(app, item, NULL);
-	if (item->flags & (ITEM_FLAG_APPDIR | ITEM_FLAG_EXEC_FILE))
+	if (item->flags & ITEM_FLAG_APPDIR || EXECUTABLE_FILE(item))
 	{
 		guchar	*path;
 
@@ -778,6 +681,7 @@ out:
 
 /* Find the current command which is used to run files of this type,
  * and return a textual description of it.
+ * Only call for non-executable files.
  * g_free() the result.
  */
 gchar *describe_current_command(MIME_type *type)
@@ -789,9 +693,6 @@ gchar *describe_current_command(MIME_type *type)
 
 	g_return_val_if_fail(type != NULL, NULL);
 
-	if (type == application_executable)
-		return g_strdup(_("Execute file"));
-
 	handler = handler_for(type);
 
 	if (!handler)
@@ -1025,7 +926,7 @@ static char *get_action_save_path(GtkWidget *dialog)
 		type_name = g_strconcat(type->media_type, "_",
 				type->subtype, NULL);
 
-	path = choices_find_path_save("", PROJECT, FALSE);
+	path = choices_find_xdg_path_save("", PROJECT, SITE, FALSE);
 	if (!path)
 	{
 		report_error(
@@ -1034,7 +935,7 @@ static char *get_action_save_path(GtkWidget *dialog)
 	}
 	g_free(path);
 
-	path = choices_find_path_save(type_name, "MIME-types", TRUE);
+	path = choices_find_xdg_path_save(type_name, "MIME-types", SITE, TRUE);
 
 	if (!remove_handler_with_confirm(path))
 		null_g_free(&path);
@@ -1095,8 +996,7 @@ gboolean can_set_run_action(DirItem *item)
 {
 	g_return_val_if_fail(item != NULL, FALSE);
 
-	return item->base_type == TYPE_FILE &&
-		!(item->mime_type == application_executable);
+	return item->base_type == TYPE_FILE && !EXECUTABLE_FILE(item);
 }
 
 /* Parse file type colours and allocate/free them as necessary */
@@ -1172,8 +1072,7 @@ GdkColor *type_get_colour(DirItem *item, GdkColor *normal)
 	if (!o_display_colour_types.int_value)
 		return normal;
 
-	if (item->flags & ITEM_FLAG_EXEC_FILE &&
-	    item->mime_type == application_executable)
+	if (EXECUTABLE_FILE(item))
 		type = TYPE_EXEC;
 	else if (item->flags & ITEM_FLAG_APPDIR)
 		type = TYPE_APPDIR;
@@ -1183,104 +1082,6 @@ GdkColor *type_get_colour(DirItem *item, GdkColor *normal)
 	return &type_colours[type];
 }
 
-/* Process the 'Patterns' value */
-static void add_pattern(MIME_type *type, const char *pattern, GHashTable *globs)
-{
-	if (pattern[0] == '*' && pattern[1] == '.' &&
-	    strpbrk(pattern + 2, "*?[") == NULL)
-	{
-		g_hash_table_insert(extension_hash,
-				    g_strdup(pattern + 2),
-				    type);
-	}
-	else if (strpbrk(pattern, "*?[") == NULL)
-		g_hash_table_insert(literal_hash, g_strdup(pattern), type);
-	else
-		g_hash_table_insert(globs, g_strdup(pattern), type);
-}
-
-/* Load and parse this file. literal_hash and extension_hash are updated
- * directly. Other patterns are added to 'globs'.
- */
-static void import_file(const gchar *file, GHashTable *globs)
-{
-	MIME_type *type = NULL;
-	GError *error = NULL;
-	gchar  *data, *line;
-
-	if (access(file, F_OK) != 0)
-		return;		/* Doesn't exist. No problem. */
-
-	if (!g_file_get_contents(file, &data, NULL, &error))
-	{
-		delayed_error(_("Error loading MIME database:\n%s"),
-				error->message);
-		g_error_free(error);
-		return;
-	}
-	
-	line = data;
-
-	while (line && *line)
-	{
-		char *nl;
-
-		nl = strchr(line, '\n');
-		if (!nl)
-			break;
-		*nl = '\0';
-
-		if (*line != '#')
-		{
-			const gchar *colon;
-			gchar *name;
-
-			colon = strchr(line, ':');
-			if (!colon)
-			{
-				delayed_error(_("File '%s' corrupted!"), file);
-				break;
-			}
-
-			name = g_strndup(line, colon - line);
-			type = get_mime_type(name, TRUE);
-			g_free(name);
-			if (!type)
-				g_warning("Invalid type in '%s'", file);
-
-			add_pattern(type, colon + 1, globs);
-		}
-
-		line = nl + 1;
-	}
-
-	g_free(data);
-}
-
-static void add_to_glob_patterns(gpointer key, gpointer value, gpointer unused)
-{
-	Pattern *pattern;
-
-	pattern = g_new(Pattern, 1);
-	pattern->glob = g_strdup((gchar *) key);
-	pattern->type = (MIME_type *) value;
-	pattern->len = strlen(pattern->glob);
-
-	g_ptr_array_add(glob_patterns, pattern);
-}
-
-static gint sort_by_strlen(gconstpointer a, gconstpointer b)
-{
-	const Pattern *pa = *(const Pattern **) a;
-	const Pattern *pb = *(const Pattern **) b;
-
-	if (pa->len > pb->len)
-		return -1;
-	else if (pa->len == pb->len)
-		return 0;
-	return 1;
-}
-
 static char **get_xdg_data_dirs(int *n_dirs)
 {
 	const char *env;
@@ -1301,96 +1102,11 @@ static char **get_xdg_data_dirs(int *n_dirs)
 		dirs[0] = g_strdup(env);
 	else
 		dirs[0] = g_build_filename(g_get_home_dir(), ".local",
-						"share", NULL);
+					   "share", NULL);
 	*n_dirs = n + 1;
 	return dirs;
 }
 
-/* Clear all currently stored information and re-read everything.
- * Note: calls filer_update_all.
- */
-static void load_mime_types(void)
-{
-	GHashTable *globs;
-	char **dirs;
-	int n_dirs;
-	int i;
-
-	dirs = get_xdg_data_dirs(&n_dirs);
-	g_return_if_fail(dirs != NULL);
-
-	{
-		struct stat info;
-		if (lstat(make_path(home_dir, ".mime"), &info) == 0 &&
-		    S_ISDIR(info.st_mode))
-		{
-			delayed_error(_("The ~/.mime directory has moved. "
-				"It should now be ~/.local/share/mime. You "
-				"should move it there (and make a symlink "
-				"from ~/.mime to it for older applications)."));
-		}
-	}
-
-	if (!glob_patterns)
-		glob_patterns = g_ptr_array_new();
-	else
-	{
-		int i;
-
-		for (i = glob_patterns->len - 1; i >= 0; i--)
-		{
-			Pattern *p = glob_patterns->pdata[i];
-			g_free(p->glob);
-			g_free(p);
-		}
-		g_ptr_array_set_size(glob_patterns, 0);
-	}
-
-	if (literal_hash)
-		g_hash_table_destroy(literal_hash);
-	if (extension_hash)
-		g_hash_table_destroy(extension_hash);
-	literal_hash = g_hash_table_new_full(g_str_hash, g_str_equal,
-						g_free, NULL);
-	extension_hash = g_hash_table_new_full(g_str_hash, g_str_equal,
-						g_free, NULL);
-	globs = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, NULL);
-
-	for (i = n_dirs - 1; i >= 0; i--)
-	{
-		char *path;
-		path = g_build_filename(dirs[i], "mime", "globs", NULL);
-		import_file(path, globs);
-		g_free(path);
-		g_free(dirs[i]);
-	}
-	g_free(dirs);
-
-	/* Turn the globs hash into a pointer array */
-	g_hash_table_foreach(globs, add_to_glob_patterns, NULL);
-	g_hash_table_destroy(globs);
-	globs = NULL;
-
-	if (glob_patterns->len)
-		g_ptr_array_sort(glob_patterns, sort_by_strlen);
-
-	if (g_hash_table_size(extension_hash) == 0)
-	{
-		delayed_error(_("The standard MIME type database "
-			"(version 0.9 or later) was not found. "
-			"The filer will probably not show the correct "
-			"types for different files. You should download and "
-			"install the 'shared-mime-info-0.9' package from "
-			"here:\n"
-		"http://www.freedesktop.org/software/shared-mime-info\n\n"
-		"If you have already installed this package, check that the "
-		"permissions allow the files to be read (check "
-		"/usr/local/share/mime/globs or /usr/share/mime/globs)."));
-	}
-
-	filer_update_all();
-}
-
 /* Try to fill in 'type->comment' from this document */
 static void get_comment(MIME_type *type, const guchar *path)
 {
@@ -1419,7 +1135,7 @@ static void get_comment(MIME_type *type, const guchar *path)
 static void find_comment(MIME_type *type)
 {
 	char **dirs;
-	int i, n_dirs;
+	int i, n_dirs = 0;
 
 	if (type->comment)
 	{
@@ -1461,7 +1177,7 @@ const char *mime_type_comment(MIME_type *type)
 
 static void set_icon_theme(void)
 {
-	RoxIconInfo *info;
+	GtkIconInfo *info;
 	char *icon_home;
 	const char *theme_name = o_icon_theme.value;
 
@@ -1470,19 +1186,19 @@ static void set_icon_theme(void)
 
 	while (1)
 	{
-		rox_icon_theme_set_custom_theme(icon_theme, theme_name);
-		info = rox_icon_theme_lookup_icon(icon_theme,
+		gtk_icon_theme_set_custom_theme(icon_theme, theme_name);
+		info = gtk_icon_theme_lookup_icon(icon_theme,
 				"mime-application:postscript",
 				ICON_HEIGHT, 0);
 		if (!info)
 		{
-			info = rox_icon_theme_lookup_icon(icon_theme,
+			info = gtk_icon_theme_lookup_icon(icon_theme,
 					"gnome-mime-application-postscript",
 					ICON_HEIGHT, 0);
 		}
 		if (info)
 		{
-			rox_icon_info_free(info);
+			gtk_icon_info_free(info);
 			return;
 		}
 
@@ -1503,13 +1219,17 @@ static void set_icon_theme(void)
 
 	icon_home = g_build_filename(home_dir, ".icons", "ROX", NULL);
 	if (symlink(make_path(app_dir, "ROX"), icon_home))
+	{
 		delayed_error(_("Failed to create symlink '%s':\n%s\n\n"
 		"(this may mean that the ROX theme already exists there, but "
 		"the 'mime-application:postscript' icon couldn't be loaded for "
-		"some reason)"), icon_home, g_strerror(errno));
+		"some reason, or %s is a link to an invalid directory; try "
+		"deleting it)"), icon_home, g_strerror(errno), icon_home);
+		open_to_show(icon_home);
+	}
 	g_free(icon_home);
 
-	rox_icon_theme_rescan_if_needed(icon_theme);
+	gtk_icon_theme_rescan_if_needed(icon_theme);
 }
 
 static guchar *read_theme(Option *option)
@@ -1609,7 +1329,7 @@ static GList *build_icon_theme(Option *option, xmlNode *node, guchar *label)
 	menu = gtk_menu_new();
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(button), menu);
 
-	rox_icon_theme_get_search_path(icon_theme, &theme_dirs, &n_dirs);
+	gtk_icon_theme_get_search_path(icon_theme, &theme_dirs, &n_dirs);
 	names = g_ptr_array_new();
 	for (i = 0; i < n_dirs; i++)
 		add_themes_from_dir(names, theme_dirs[i]);
