@@ -1,8 +1,8 @@
 /*
- * $Id: view_details.c,v 1.43 2004/04/13 09:04:07 tal197 Exp $
+ * $Id: view_details.c,v 1.47 2005/12/15 18:39:54 tal197 Exp $
  *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2003, the ROX-Filer team.
+ * Copyright (C) 2005, the ROX-Filer team.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -377,16 +377,22 @@ static void details_get_value(GtkTreeModel *tree_model,
 			break;
 		case COL_TYPE:
 			g_value_init(value, G_TYPE_STRING);
-			g_value_set_string(value, 
-				item->flags & ITEM_FLAG_APPDIR? "App" :
-				S_ISDIR(m) ? "Dir" :
-				S_ISCHR(m) ? "Char" :
-				S_ISBLK(m) ? "Blck" :
-				S_ISLNK(m) ? "Link" :
-				S_ISSOCK(m) ? "Sock" :
-				S_ISFIFO(m) ? "Pipe" :
-				S_ISDOOR(m) ? "Door" :
-				"File");
+			if(o_display_show_full_type.int_value)
+				g_value_set_string(value, 
+						   item->flags & ITEM_FLAG_APPDIR? "Application" :
+						   mime_type_comment(item->mime_type));
+			else
+				g_value_set_string(value, 
+						   item->flags & ITEM_FLAG_APPDIR? "App" :
+						   S_ISDIR(m) ? "Dir" :
+						   S_ISCHR(m) ? "Char" :
+						   S_ISBLK(m) ? "Blck" :
+						   S_ISLNK(m) ? "Link" :
+						   S_ISSOCK(m) ? "Sock" :
+						   S_ISFIFO(m) ? "Pipe" :
+						   S_ISDOOR(m) ? "Door" :
+						   "File");
+			
 			break;
 		case COL_WEIGHT:
 			g_value_init(value, G_TYPE_INT);
@@ -659,6 +665,9 @@ static int get_lasso_index(ViewDetails *view_details, int y)
 	gint cell_y;
 	int i;
 
+	if (y < 0)
+		y = 0;	/* gtk_tree_view_get_path_at_pos can't handle negatives */
+
 	if (gtk_tree_view_get_path_at_pos(tree, 4, y, &path,
 				&column, NULL, &cell_y))
 	{
@@ -981,14 +990,17 @@ static void view_details_init(GTypeInstance *object, gpointer gclass)
 				GTK_SELECTION_MULTIPLE);
 	gtk_tree_selection_set_select_function(view_details->selection,
 			test_can_change_selection, view_details, NULL);
-	g_signal_connect(view_details->selection, "changed",
-			G_CALLBACK(selection_changed), view_details);
 
 	/* Sorting */
 	view_details->sort_fn = NULL;
 	sortable_list = GTK_TREE_SORTABLE(object);
 
 	gtk_tree_view_set_model(treeview, GTK_TREE_MODEL(view_details));
+	/* Do this after set_model, because that can generate this
+	 * signal...
+	 */
+	g_signal_connect(view_details->selection, "changed",
+			G_CALLBACK(selection_changed), view_details);
 
 	/* Icon */
 	cell = cell_icon_new(view_details);
@@ -1002,6 +1014,7 @@ static void view_details_init(GTypeInstance *object, gpointer gclass)
 	gtk_tree_view_column_set_resizable(column, TRUE);
 	ADD_TEXT_COLUMN(_("_Type"), COL_TYPE);
 	gtk_tree_view_column_set_sort_column_id(column, COL_TYPE);
+	gtk_tree_view_column_set_resizable(column, TRUE);
 	ADD_TEXT_COLUMN(_("_Permissions"), COL_PERM);
 	ADD_TEXT_COLUMN(_("_Owner"), COL_OWNER);
 	gtk_tree_view_column_set_sort_column_id(column, COL_OWNER);
