@@ -42,7 +42,9 @@ confirm_or_die() {
 cd `dirname $0`
 
 echo Testing the filer...
-./ROX-Filer/AppRun -v
+# Note: must use full path due to bug in xterm
+# http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=342782
+"`pwd`/ROX-Filer/AppRun" -v
 if [ $? -ne 0 ]; then
 	die "Filer doesn't work! Giving up..."
 fi
@@ -74,22 +76,24 @@ case $REPLY in
 	1) APPDIR=/usr/local/apps
 	   BINDIR=/usr/local/bin
 	   SHAREDIR=/usr/local/share
-	   CHOICESDIR=${SHAREDIR}/Choices
+	   #CHOICESDIR=${SHAREDIR}/Choices
+	   XDGDATADIR=/usr/local/share
+	   CHOICESDIR=/etc/xdg
 	   MANDIR=/usr/local/man	# (not under share!)
 	   ;;
 	2) APPDIR=${HOME}/Apps
 	   BINDIR=${HOME}/bin
-	   if [ -n "$CHOICESPATH" ]; then
-		CHOICESDIR=`echo $CHOICESPATH | sed -e 's/^\([^:]*\).*$/\\1/'`
-	   fi
-	   if [ ! -n "$CHOICESDIR" ]; then
-	   	CHOICESDIR=${HOME}/Choices
-	   fi
 	   if [ -n "$XDG_DATA_HOME" ]; then
 	        SHAREDIR=${XDG_DATA_HOME}
 	   else
 	   	SHAREDIR=${HOME}/.local/share
 	   fi
+	   if [ -n "$XDG_CONFIG_HOME" ]; then
+	        CHOICESDIR=${XDG_CONFIG_HOME}
+	   else
+	   	CHOICESDIR=${HOME}/.config
+	   fi
+	   
 	   MANDIR=${HOME}/man
 	   if [ ! -d "$MANDIR" ]; then
 	     MANDIR=
@@ -98,7 +102,7 @@ case $REPLY in
 	3) APPDIR=/usr/apps
 	   BINDIR=/usr/bin
 	   SHAREDIR=/usr/share
-	   CHOICESDIR=${SHAREDIR}/Choices
+	   CHOICESDIR=/etc/xdg
 	   MANDIR=${SHAREDIR}/man
 	   ;;
 	4) echo "Where should the ROX-Filer application go?"
@@ -114,7 +118,7 @@ case $REPLY in
 	   SHAREDIR="$DIR"
 	   echo
 	   echo "Where should the default run actions go?"
-	   get_dir "$SHAREDIR/Choices"
+	   get_dir "/etc/xdg"
 	   CHOICESDIR="$DIR"
 	   echo
 	   echo "Where should the man pages go?"
@@ -126,7 +130,7 @@ case $REPLY in
            APPDIR="$DIR/apps"
 	   BINDIR="$DIR/bin"
 	   SHAREDIR="$DIR/share"
-	   CHOICESDIR=${SHAREDIR}/Choices
+	   CHOICESDIR=/etc/xdg
 	   MANDIR="$DIR/man"	# (not under share!)
 	   ;;
 
@@ -185,11 +189,11 @@ if [ -n "$MANDIR" ]; then
 fi
 
 echo "Installing run actions (existing actions will not be replaced)..."
-endir "$CHOICESDIR/MIME-types"
+endir "$CHOICESDIR/rox.sourceforge.net/MIME-types"
 cd Choices || die "Choices missing"
 for file in MIME-types/*; do
   if [ -f "$file" ]; then
-    dest="$CHOICESDIR/$file"
+    dest="$CHOICESDIR/rox.sourceforge.net/$file"
     if [ ! -f "$dest" ]; then
       if [ ! -d "$dest" ]; then
         echo Install $file as $dest
@@ -205,7 +209,7 @@ endir "$MIMEDIR/packages"
 cp rox.xml "$MIMEINFO" || die "Failed to create $MIMEINFO"
 update-mime-database "$MIMEDIR" || die "update-mime-database failed
 Make sure you have installed the shared MIME database from:
-http://www.freedesktop.org/software/shared-mime-info"
+http://www.freedesktop.org/wiki/Software_2fshared_2dmime_2dinfo"
 
 echo "Installing application..."
 endir "$APPDIR"
@@ -218,8 +222,15 @@ if [ -d "$APPDIR/ROX-Filer" ]; then
 	echo Deleting...
 	# Move it away first.  If $APPDIR is on an nfs mount then deleting
 	# the binary may fail.
-	mv "$APPDIR/ROX-Filer" "$APPDIR/.ROX-Filer"
-	rm -rf "$APPDIR/.ROX-Filer"
+	old="$APPDIR/.ROX-Filer.old"
+	if [ -d "$old" ]; then
+	    echo "Cleaning up old copy from previous install."
+	    rm -rf "$old"
+	fi
+	mv "$APPDIR/ROX-Filer" "$old"
+	rm -rf "$old" || echo "Could not delete old copy.
+This is usually because it is still executing.  When it exits you may like 
+to delete $old"
 fi
 cp -r ROX-Filer "$APPDIR"
 
